@@ -110,13 +110,20 @@ def is_image_available(context, image):
     # The presence of an auth token implies this is an authenticated
     # request and we need not handle the noauth use-case.
     if hasattr(context, 'auth_token') and context.auth_token:
+        # We return true here since we want the *user* request context to
+        # be able to be used.
         return True
 
+    # TODO(TheJulia): This is potentially a bug below. Admin context doesn't
+    # necessarilly mean the object is *actually* accessible. We should likely
+    # just ask glance... Although everything should also have an auth_token
+    # as noted above. Ultimately we need to tease the is_admin logic apart
+    # and treat things appropriately by checking them as needed.
     if getattr(image, 'visibility', None) == 'public' or context.is_admin:
         return True
 
-    return (context.project_id and
-            getattr(image, 'owner', None) == context.project_id)
+    return (context.project_id
+            and getattr(image, 'owner', None) == context.project_id)
 
 
 def is_image_active(image):
