@@ -284,8 +284,26 @@ RELEASE_MAPPING = {
         }
     },
     '16.2': {
-        'api': '1.68',
-        'rpc': '1.51',
+        'api': '1.69',
+        'rpc': '1.52',
+        'objects': {
+            'Allocation': ['1.1'],
+            'Node': ['1.35'],
+            'Conductor': ['1.3'],
+            'Chassis': ['1.3'],
+            'Deployment': ['1.0'],
+            'DeployTemplate': ['1.1'],
+            'Port': ['1.10'],
+            'Portgroup': ['1.4'],
+            'Trait': ['1.0'],
+            'TraitList': ['1.0'],
+            'VolumeConnector': ['1.0'],
+            'VolumeTarget': ['1.0'],
+        }
+    },
+    '17.0': {
+        'api': '1.72',
+        'rpc': '1.54',
         'objects': {
             'Allocation': ['1.1'],
             'Node': ['1.35'],
@@ -302,8 +320,8 @@ RELEASE_MAPPING = {
         }
     },
     'master': {
-        'api': '1.69',
-        'rpc': '1.52',
+        'api': '1.72',
+        'rpc': '1.54',
         'objects': {
             'Allocation': ['1.1'],
             'Node': ['1.35'],
@@ -334,9 +352,9 @@ RELEASE_MAPPING = {
 #
 #            There should be at most two named mappings here.
 
-# NOTE(mgoddard): remove Ussuri prior to the Wallaby release.
-RELEASE_MAPPING['ussuri'] = RELEASE_MAPPING['15.0']
+# NOTE(mgoddard): remove victoria prior to the xena release.
 RELEASE_MAPPING['victoria'] = RELEASE_MAPPING['16.0']
+RELEASE_MAPPING['wallaby'] = RELEASE_MAPPING['17.0']
 
 # List of available versions with named versions first; 'master' is excluded.
 RELEASE_VERSIONS = sorted(set(RELEASE_MAPPING) - {'master'}, reverse=True)
