@@ -93,7 +93,7 @@ def find_step(steps, step):
 
 
 def _get_steps(task, interfaces, get_method, enabled=False,
-               sort_step_key=None):
+               sort_step_key=None, prio_overrides=None):
     """Get steps for task.node.
 
     :param task: A TaskManager object
@@ -108,6 +108,10 @@ def _get_steps(task, interfaces, get_method, enabled=False,
     :param sort_step_key: If set, this is a method (key) used to sort the steps
         from highest priority to lowest priority. For steps having the same
         priority, they are sorted from highest interface priority to lowest.
+    :param prio_overrides: An optional dictionary of priority overrides for
+        steps, e.g:
+        {'deploy.erase_devices_metadata': '123',
+         'management.reset_bios_to_default': '234'}
     :raises: NodeCleaningFailure or InstanceDeployFailure if there was a
         problem getting the steps.
     :returns: A list of step dictionaries
@@ -120,6 +124,12 @@ def _get_steps(task, interfaces, get_method, enabled=False,
             interface_steps = [x for x in getattr(interface, get_method)(task)
                                if not enabled or x['priority'] > 0]
             steps.extend(interface_steps)
+    if prio_overrides is not None:
+        for step in steps:
+            override_key = '%(interface)s.%(step)s' % step
+            override_value = prio_overrides.get(override_key)
+            if override_value:
+                step["priority"] = int(override_value)
     if sort_step_key:
         steps = _sorted_steps(steps, sort_step_key)
     return steps
@@ -139,8 +149,24 @@ def _get_cleaning_steps(task, enabled=False, sort=True):
     :returns: A list of clean step dictionaries
     """
     sort_key = _clean_step_key if sort else None
-    return _get_steps(task, CLEANING_INTERFACE_PRIORITY, 'get_clean_steps',
-                      enabled=enabled, sort_step_key=sort_key)
+    if CONF.conductor.clean_step_priority_override:
+        csp_override = {}
+        for element in CONF.conductor.clean_step_priority_override:
+            csp_override.update(element)
+
+        cleaning_steps = _get_steps(task, CLEANING_INTERFACE_PRIORITY,
+                                    'get_clean_steps', enabled=enabled,
+                                    sort_step_key=sort_key,
+                                    prio_overrides=csp_override)
+
+        LOG.debug("cleaning_steps after applying "
+                  "clean_step_priority_override for node %(node)s: %(step)s",
+                  task.node.uuid, cleaning_steps)
+    else:
+        cleaning_steps = _get_steps(task, CLEANING_INTERFACE_PRIORITY,
+                                    'get_clean_steps', enabled=enabled,
+                                    sort_step_key=sort_key)
+    return cleaning_steps
 
 
 def _get_deployment_steps(task, enabled=False, sort=True):
@@ -161,13 +187,15 @@ def _get_deployment_steps(task, enabled=False, sort=True):
                       enabled=enabled, sort_step_key=sort_key)
 
 
-def set_node_cleaning_steps(task):
+def set_node_cleaning_steps(task, disable_ramdisk=False):
     """Set up the node with clean step information for cleaning.
 
     For automated cleaning, get the clean steps from the driver.
     For manual cleaning, the user's clean steps are known but need to be
     validated against the driver's clean steps.
 
+    :param disable_ramdisk: If `True`, only steps with requires_ramdisk=False
+        are accepted.
     :raises: InvalidParameterValue if there is a problem with the user's
              clean steps.
     :raises: NodeCleaningFailure if there was a problem getting the
@@ -190,8 +218,8 @@ def set_node_cleaning_steps(task):
         # Now that we know what the driver's available clean steps are, we can
         # do further checks to validate the user's clean steps.
         steps = node.driver_internal_info['clean_steps']
-        driver_internal_info['clean_steps'] = (
-            _validate_user_clean_steps(task, steps))
+        driver_internal_info['clean_steps'] = _validate_user_clean_steps(
+            task, steps, disable_ramdisk=disable_ramdisk)
 
     node.clean_step = {}
     driver_internal_info['clean_step_index'] = None
@@ -382,7 +410,8 @@ def _validate_deploy_steps_unique(user_steps):
     return errors
 
 
-def _validate_user_step(task, user_step, driver_step, step_type):
+def _validate_user_step(task, user_step, driver_step, step_type,
+                        disable_ramdisk=False):
     """Validate a user-specified step.
 
     :param task: A TaskManager object
@@ -424,6 +453,8 @@ def _validate_user_step(task, user_step, driver_step, step_type):
                                'required': False } } }
 
     :param step_type: either 'clean' or 'deploy'.
+    :param disable_ramdisk: If `True`, only steps with requires_ramdisk=False
+        are accepted. Only makes sense for manual cleaning at the moment.
     :return: a list of validation error strings for the step.
     """
     errors = []
@@ -453,6 +484,9 @@ def _validate_user_step(task, user_step, driver_step, step_type):
                      {'type': step_type, 'step': user_step,
                       'miss': ', '.join(missing)})
             errors.append(error)
+        if disable_ramdisk and driver_step.get('requires_ramdisk', True):
+            error = _('clean step %s requires booting a ramdisk') % user_step
+            errors.append(error)
 
     if step_type == 'clean':
         # Copy fields that should not be provided by a user
@@ -477,7 +511,8 @@ def _validate_user_step(task, user_step, driver_step, step_type):
 
 
 def _validate_user_steps(task, user_steps, driver_steps, step_type,
-                         error_prefix=None, skip_missing=False):
+                         error_prefix=None, skip_missing=False,
+                         disable_ramdisk=False):
     """Validate the user-specified steps.
 
     :param task: A TaskManager object
@@ -522,6 +557,9 @@ def _validate_user_steps(task, user_steps, driver_steps, step_type,
     :param step_type: either 'clean' or 'deploy'.
     :param error_prefix: String to use as a prefix for exception messages, or
         None.
+    :param skip_missing: Whether to silently ignore unknown steps.
+    :param disable_ramdisk: If `True`, only steps with requires_ramdisk=False
+        are accepted. Only makes sense for manual cleaning at the moment.
     :raises: InvalidParameterValue if validation of steps fails.
     :raises: NodeCleaningFailure or InstanceDeployFailure if
         there was a problem getting the steps from the driver.
@@ -554,7 +592,7 @@ def _validate_user_steps(task, user_steps, driver_steps, step_type,
             continue
 
         step_errors = _validate_user_step(task, user_step, driver_step,
-                                          step_type)
+                                          step_type, disable_ramdisk)
         errors.extend(step_errors)
         result.append(user_step)
 
@@ -572,7 +610,7 @@ def _validate_user_steps(task, user_steps, driver_steps, step_type,
     return result
 
 
-def _validate_user_clean_steps(task, user_steps):
+def _validate_user_clean_steps(task, user_steps, disable_ramdisk=False):
     """Validate the user-specified clean steps.
 
     :param task: A TaskManager object
@@ -588,13 +626,16 @@ def _validate_user_clean_steps(task, user_steps):
               { 'interface': 'deploy',
                 'step': 'upgrade_firmware',
                 'args': {'force': True} }
+    :param disable_ramdisk: If `True`, only steps with requires_ramdisk=False
+        are accepted.
     :raises: InvalidParameterValue if validation of clean steps fails.
     :raises: NodeCleaningFailure if there was a problem getting the
         clean steps from the driver.
     :return: validated clean steps update with information from the driver
     """
     driver_steps = _get_cleaning_steps(task, enabled=False, sort=False)
-    return _validate_user_steps(task, user_steps, driver_steps, 'clean')
+    return _validate_user_steps(task, user_steps, driver_steps, 'clean',
+                                disable_ramdisk=disable_ramdisk)
 
 
 def _validate_user_deploy_steps(task, user_steps, error_prefix=None,
