@@ -10,29 +10,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import itertools
-
 from oslo_log import log
 
 import ironic.conf
 
-_default_opt_lists = [
-    ironic.conf.default.api_opts,
-    ironic.conf.default.driver_opts,
-    ironic.conf.default.exc_log_opts,
-    ironic.conf.default.hash_opts,
-    ironic.conf.default.image_opts,
-    ironic.conf.default.img_cache_opts,
-    ironic.conf.default.netconf_opts,
-    ironic.conf.default.notification_opts,
-    ironic.conf.default.path_opts,
-    ironic.conf.default.portgroup_opts,
-    ironic.conf.default.service_opts,
-    ironic.conf.default.utils_opts,
-]
 
 _opts = [
-    ('DEFAULT', itertools.chain(*_default_opt_lists)),
+    ('DEFAULT', ironic.conf.default.list_opts()),
     ('agent', ironic.conf.agent.opts),
     ('ansible', ironic.conf.ansible.opts),
     ('api', ironic.conf.api.opts),
@@ -51,12 +35,14 @@ _opts = [
     ('ipmi', ironic.conf.ipmi.opts),
     ('irmc', ironic.conf.irmc.opts),
     ('iscsi', ironic.conf.iscsi.opts),
-    ('json_rpc', ironic.conf.json_rpc.list_opts()),
+    ('anaconda', ironic.conf.anaconda.opts),
     ('metrics', ironic.conf.metrics.opts),
     ('metrics_statsd', ironic.conf.metrics_statsd.opts),
+    ('molds', ironic.conf.molds.opts),
     ('neutron', ironic.conf.neutron.list_opts()),
     ('nova', ironic.conf.nova.list_opts()),
     ('pxe', ironic.conf.pxe.opts),
+    ('redfish', ironic.conf.redfish.opts),
     ('service_catalog', ironic.conf.service_catalog.list_opts()),
     ('snmp', ironic.conf.snmp.opts),
     ('swift', ironic.conf.swift.list_opts()),
@@ -96,10 +82,10 @@ def update_opt_defaults():
             'eventlet.wsgi.server=INFO',
             'iso8601=WARNING',
             'requests=WARNING',
-            'neutronclient=WARNING',
             'glanceclient=WARNING',
             'urllib3.connectionpool=WARNING',
             'keystonemiddleware.auth_token=INFO',
             'keystoneauth.session=INFO',
+            'openstack=WARNING',
         ]
     )
