@@ -162,6 +162,27 @@ def add_port_filter_by_node_project(query, value):
                         | (models.Node.lessee == value))
 
 
+def add_portgroup_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.Portgroup.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
+def add_volume_conn_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.VolumeConnector.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
+def add_volume_target_filter_by_node_project(query, value):
+    query = query.join(models.Node,
+                       models.VolumeTarget.node_id == models.Node.id)
+    return query.filter((models.Node.owner == value)
+                        | (models.Node.lessee == value))
+
+
 def add_portgroup_filter(query, value):
     """Adds a portgroup-specific filter to a query.
 
@@ -418,7 +439,7 @@ class Connection(api.Connection):
         return _paginate_query(models.Node, limit, marker,
                                sort_key, sort_dir, query)
 
-    def check_node_list(self, idents):
+    def check_node_list(self, idents, project=None):
         mapping = {}
         if idents:
             idents = set(idents)
@@ -438,6 +459,10 @@ class Connection(api.Connection):
             sql.or_(models.Node.uuid.in_(uuids),
                     models.Node.name.in_(names))
         )
+        if project:
+            query = query.filter((models.Node.owner == project)
+                                 | (models.Node.lessee == project))
+
         for row in query:
             if row[0] in idents:
                 mapping[row[0]] = row[0]
@@ -796,8 +821,10 @@ class Connection(api.Connection):
             if count == 0:
                 raise exception.PortNotFound(port=port_id)
 
-    def get_portgroup_by_id(self, portgroup_id):
+    def get_portgroup_by_id(self, portgroup_id, project=None):
         query = model_query(models.Portgroup).filter_by(id=portgroup_id)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         try:
             return query.one()
         except NoResultFound:
@@ -810,8 +837,10 @@ class Connection(api.Connection):
         except NoResultFound:
             raise exception.PortgroupNotFound(portgroup=portgroup_uuid)
 
-    def get_portgroup_by_address(self, address):
+    def get_portgroup_by_address(self, address, project=None):
         query = model_query(models.Portgroup).filter_by(address=address)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         try:
             return query.one()
         except NoResultFound:
@@ -825,14 +854,19 @@ class Connection(api.Connection):
             raise exception.PortgroupNotFound(portgroup=name)
 
     def get_portgroup_list(self, limit=None, marker=None,
-                           sort_key=None, sort_dir=None):
+                           sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.Portgroup)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         return _paginate_query(models.Portgroup, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_portgroups_by_node_id(self, node_id, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None, project=None):
         query = model_query(models.Portgroup)
         query = query.filter_by(node_id=node_id)
+        if project:
+            query = add_portgroup_filter_by_node_project(query, project)
         return _paginate_query(models.Portgroup, limit, marker,
                                sort_key, sort_dir, query)
 
@@ -1110,26 +1144,20 @@ class Connection(api.Connection):
         return query.all()
 
     @oslo_db_api.retry_on_deadlock
-    def register_conductor_hardware_interfaces(self, conductor_id,
-                                               hardware_type, interface_type,
-                                               interfaces, default_interface):
+    def register_conductor_hardware_interfaces(self, conductor_id, interfaces):
         with _session_for_write() as session:
             try:
                 for iface in interfaces:
                     conductor_hw_iface = models.ConductorHardwareInterfaces()
                     conductor_hw_iface['conductor_id'] = conductor_id
-                    conductor_hw_iface['hardware_type'] = hardware_type
-                    conductor_hw_iface['interface_type'] = interface_type
-                    conductor_hw_iface['interface_name'] = iface
-                    is_default = (iface == default_interface)
-                    conductor_hw_iface['default'] = is_default
+                    for k, v in iface.items():
+                        conductor_hw_iface[k] = v
                     session.add(conductor_hw_iface)
                 session.flush()
-            except db_exc.DBDuplicateEntry:
-                raise exception.ConductorHardwareInterfacesAlreadyRegistered(
-                    hardware_type=hardware_type,
-                    interface_type=interface_type,
-                    interfaces=interfaces)
+            except db_exc.DBDuplicateEntry as e:
+                r = exception.ConductorHardwareInterfacesAlreadyRegistered(
+                    row=str(e.inner_exception.params))
+                raise r
 
     @oslo_db_api.retry_on_deadlock
     def unregister_conductor_hardware_interfaces(self, conductor_id):
@@ -1225,9 +1253,12 @@ class Connection(api.Connection):
                 % addresses)
 
     def get_volume_connector_list(self, limit=None, marker=None,
-                                  sort_key=None, sort_dir=None):
+                                  sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.VolumeConnector)
+        if project:
+            query = add_volume_conn_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeConnector, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_volume_connector_by_id(self, db_id):
         query = model_query(models.VolumeConnector).filter_by(id=db_id)
@@ -1246,8 +1277,10 @@ class Connection(api.Connection):
 
     def get_volume_connectors_by_node_id(self, node_id, limit=None,
                                          marker=None, sort_key=None,
-                                         sort_dir=None):
+                                         sort_dir=None, project=None):
         query = model_query(models.VolumeConnector).filter_by(node_id=node_id)
+        if project:
+            add_volume_conn_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeConnector, limit, marker,
                                sort_key, sort_dir, query)
 
@@ -1305,9 +1338,12 @@ class Connection(api.Connection):
                 raise exception.VolumeConnectorNotFound(connector=ident)
 
     def get_volume_target_list(self, limit=None, marker=None,
-                               sort_key=None, sort_dir=None):
+                               sort_key=None, sort_dir=None, project=None):
+        query = model_query(models.VolumeTarget)
+        if project:
+            query = add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker,
-                               sort_key, sort_dir)
+                               sort_key, sort_dir, query)
 
     def get_volume_target_by_id(self, db_id):
         query = model_query(models.VolumeTarget).filter_by(id=db_id)
@@ -1324,15 +1360,20 @@ class Connection(api.Connection):
             raise exception.VolumeTargetNotFound(target=uuid)
 
     def get_volume_targets_by_node_id(self, node_id, limit=None, marker=None,
-                                      sort_key=None, sort_dir=None):
+                                      sort_key=None, sort_dir=None,
+                                      project=None):
         query = model_query(models.VolumeTarget).filter_by(node_id=node_id)
+        if project:
+            add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker, sort_key,
                                sort_dir, query)
 
     def get_volume_targets_by_volume_id(self, volume_id, limit=None,
                                         marker=None, sort_key=None,
-                                        sort_dir=None):
+                                        sort_dir=None, project=None):
         query = model_query(models.VolumeTarget).filter_by(volume_id=volume_id)
+        if project:
+            query = add_volume_target_filter_by_node_project(query, project)
         return _paginate_query(models.VolumeTarget, limit, marker, sort_key,
                                sort_dir, query)
 
