@@ -31,7 +31,6 @@ from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
 from ironic.drivers.modules import agent_base
-from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 
@@ -270,8 +269,6 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
                         'step deploy.write_image, using the deprecated '
                         'synchronous fall-back', task.node.uuid)
 
-        client = agent_client.get_client(task)
-
         if self.has_decomposed_deploy_steps and has_write_image:
             configdrive = node.instance_info.get('configdrive')
             # Now switch into the corresponding in-band deploy step and let the
@@ -281,10 +278,10 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
                         'args': {'image_info': image_info,
                                  'configdrive': configdrive}}
             return agent_base.execute_step(task, new_step, 'deploy',
-                                           client=client)
+                                           client=self._client)
         else:
             # TODO(dtantsur): remove in W
-            command = client.prepare_image(node, image_info, wait=True)
+            command = self._client.prepare_image(node, image_info, wait=True)
             if command['command_status'] == 'FAILED':
                 # TODO(jimrollenhagen) power off if using neutron dhcp to
                 #                      align with pxe driver?
@@ -295,8 +292,8 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
 
     # TODO(dtantsur): remove in W
     def _get_uuid_from_result(self, task, type_uuid):
-        client = agent_client.get_client(task)
-        command = client.get_last_command_status(task.node, 'prepare_image')
+        command = self._client.get_last_command_status(task.node,
+                                                       'prepare_image')
         if (not command
                 or not command.get('command_result', {}).get('result')):
             msg = _('Unexpected response from the agent for node %s: the '
@@ -350,9 +347,8 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
         # ppc64* hardware we need to provide the 'PReP_Boot_partition_uuid' to
         # direct where the bootloader should be installed.
         driver_internal_info = task.node.driver_internal_info
-        client = agent_client.get_client(task)
         try:
-            partition_uuids = client.get_partition_uuids(node).get(
+            partition_uuids = self._client.get_partition_uuids(node).get(
                 'command_result') or {}
             root_uuid = partition_uuids.get('root uuid')
         except exception.AgentAPIError:
@@ -670,6 +666,24 @@ class AgentRAID(base.RAIDInterface):
         """Return the properties of the interface."""
         return {}
 
+    @METRICS.timer('AgentRAID.get_clean_steps')
+    def get_clean_steps(self, task):
+        """Get the list of clean steps from the agent.
+
+        :param task: a TaskManager object containing the node
+        :raises NodeCleaningFailure: if the clean steps are not yet
+            available (cached), for example, when a node has just been
+            enrolled and has not been cleaned yet.
+        :returns: A list of clean step dictionaries
+        """
+        new_priorities = {
+            'delete_configuration': CONF.deploy.delete_configuration_priority,
+            'create_configuration': CONF.deploy.create_configuration_priority
+        }
+        return agent_base.get_steps(
+            task, 'clean', interface='raid',
+            override_priorities=new_priorities)
+
     @METRICS.timer('AgentRAID.get_deploy_steps')
     def get_deploy_steps(self, task):
         """Get the list of deploy steps from the agent.
@@ -812,6 +826,9 @@ class AgentRAID(base.RAIDInterface):
             returned from agent ramdisk on query of the status of command(s).
         :returns: None
         """
+        prop = task.node.properties
+        prop.pop('root_device', None)
+        task.node.properties = prop
         task.node.raid_config = {}
         task.node.save()
 
