@@ -22,7 +22,7 @@ from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import retrying
+import tenacity
 
 from ironic.common import boot_devices
 from ironic.common import dhcp_factory
@@ -612,13 +612,17 @@ class HeartbeatMixin(object):
 
     @METRICS.timer('HeartbeatMixin.heartbeat')
     def heartbeat(self, task, callback_url, agent_version,
-                  agent_verify_ca=None):
+                  agent_verify_ca=None, agent_status=None,
+                  agent_status_message=None):
         """Process a heartbeat.
 
         :param task: task to work with.
         :param callback_url: agent HTTP API URL.
         :param agent_version: The version of the agent that is heartbeating
         :param agent_verify_ca: TLS certificate for the agent.
+        :param agent_status: Status of the heartbeating agent
+        :param agent_status_message: Status message that describes the
+            agent_status
         """
         # NOTE(pas-ha) immediately skip the rest if nothing to do
         if (task.node.provision_state not in self.heartbeat_allowed_states
@@ -649,6 +653,11 @@ class HeartbeatMixin(object):
             timeutils.utcnow().isoformat())
         if agent_verify_ca:
             driver_internal_info['agent_verify_ca'] = agent_verify_ca
+        if agent_status:
+            driver_internal_info['agent_status'] = agent_status
+        if agent_status_message:
+            driver_internal_info['agent_status_message'] = \
+                agent_status_message
         node.driver_internal_info = driver_internal_info
         node.save()
 
@@ -944,6 +953,8 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                                     'step': step,
                                                     'type': step_type}))
 
+                if step_type == 'clean':
+                    step['requires_ramdisk'] = True
                 steps[step['interface']].append(step)
 
         # Save hardware manager version, steps, and date
@@ -1168,14 +1179,15 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
 
         :param task: a TaskManager object containing the node
         """
-        wait = CONF.agent.post_deploy_get_power_state_retry_interval * 1000
+        wait = CONF.agent.post_deploy_get_power_state_retry_interval
         attempts = CONF.agent.post_deploy_get_power_state_retries + 1
 
-        @retrying.retry(
-            stop_max_attempt_number=attempts,
-            retry_on_result=lambda state: state != states.POWER_OFF,
-            wait_fixed=wait
-        )
+        @tenacity.retry(stop=tenacity.stop_after_attempt(attempts),
+                        retry=(tenacity.retry_if_result(
+                            lambda state: state != states.POWER_OFF)
+                            | tenacity.retry_if_exception_type(Exception)),
+                        wait=tenacity.wait_fixed(wait),
+                        reraise=True)
         def _wait_until_powered_off(task):
             return task.driver.power.get_power_state(task)
 
@@ -1219,7 +1231,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                 'in at least %(timeout)d seconds. Forcing '
                                 'hard power off and proceeding.',
                                 {'node_uuid': node.uuid,
-                                 'timeout': (wait * (attempts - 1)) / 1000})
+                                 'timeout': (wait * (attempts - 1))})
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 # Flush the file system prior to hard rebooting the node
@@ -1340,6 +1352,10 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                 image_source = node.instance_info.get('image_source')
                 try:
                     context = task.context
+                    # TODO(TheJulia): Uhh, is_admin likely needs to be
+                    # addressed in Xena as undesirable behavior may
+                    # result, or just outright break in an entirely
+                    # system scoped configuration.
                     context.is_admin = True
                     glance = image_service.GlanceImageService(
                         context=context)
