@@ -26,11 +26,12 @@ from oslo_concurrency import processutils
 from oslo_log import log
 from oslo_utils import strutils
 from oslo_utils import units
-import retrying
+import tenacity
 import yaml
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
+from ironic.common import faults
 from ironic.common.i18n import _
 from ironic.common import images
 from ironic.common import states
@@ -40,7 +41,7 @@ from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
-from ironic.drivers.modules import agent_base_vendor as agent_base
+from ironic.drivers.modules import agent_base
 from ironic.drivers.modules import deploy_utils
 
 
@@ -375,9 +376,13 @@ def _get_clean_steps(node, interface=None, override_priorities=None):
     return steps
 
 
-class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
+class AnsibleDeploy(agent_base.HeartbeatMixin,
+                    agent_base.AgentOobStepsMixin,
+                    base.DeployInterface):
     """Interface for deploy-related actions."""
 
+    has_decomposed_deploy_steps = True
+
     def __init__(self):
         super(AnsibleDeploy, self).__init__()
         # NOTE(pas-ha) overriding agent creation as we won't be
@@ -442,12 +447,22 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         manager_utils.node_power_action(task, states.REBOOT)
         return states.DEPLOYWAIT
 
+    def process_next_step(self, task, step_type):
+        """Start the next clean/deploy step if the previous one is complete.
+
+        :param task: a TaskManager instance
+        :param step_type: "clean" or "deploy"
+        """
+        # Run the next step as soon as agent heartbeats in deploy.deploy
+        if step_type == 'deploy' and self.in_core_deploy_step(task):
+            manager_utils.notify_conductor_resume_deploy(task)
+
     @staticmethod
     def _required_image_info(task):
         """Gather and save needed image info while the context is good.
 
         Gather image info that will be needed later, during the
-        continue_deploy execution, where the context won't be the same
+        write_image execution, where the context won't be the same
         anymore, since coming from the server's heartbeat.
         """
         node = task.node
@@ -474,9 +489,12 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         # TODO(pas-ha) investigate takeover scenario
         if node.provision_state == states.DEPLOYING:
             # adding network-driver dependent provisioning ports
-            manager_utils.node_power_action(task, states.POWER_OFF)
-            power_state_to_restore = (
-                manager_utils.power_on_node_if_needed(task))
+            fast_track = manager_utils.is_fast_track(task)
+            power_state_to_restore = None
+            if not fast_track:
+                manager_utils.node_power_action(task, states.POWER_OFF)
+                power_state_to_restore = (
+                    manager_utils.power_on_node_if_needed(task))
             task.driver.network.add_provisioning_network(task)
             manager_utils.restore_power_state_if_needed(
                 task, power_state_to_restore)
@@ -562,13 +580,18 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         if not node.driver_internal_info['clean_steps']:
             # no clean steps configured, nothing to do.
             return
-        power_state_to_restore = manager_utils.power_on_node_if_needed(task)
+        fast_track = manager_utils.is_fast_track(task)
+        power_state_to_restore = None
+        if not fast_track:
+            power_state_to_restore = manager_utils.power_on_node_if_needed(
+                task)
         task.driver.network.add_cleaning_network(task)
         manager_utils.restore_power_state_if_needed(
             task, power_state_to_restore)
         boot_opt = deploy_utils.build_agent_options(node)
         task.driver.boot.prepare_ramdisk(task, boot_opt)
-        manager_utils.node_power_action(task, states.REBOOT)
+        if not fast_track:
+            manager_utils.node_power_action(task, states.REBOOT)
         return states.CLEANWAIT
 
     @METRICS.timer('AnsibleDeploy.tear_down_cleaning')
@@ -579,54 +602,52 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         :raises NodeCleaningFailure: if the cleaning ports cannot be
                 removed
         """
-        manager_utils.node_power_action(task, states.POWER_OFF)
+        fast_track = manager_utils.is_fast_track(task)
+        node = task.node
+        cleaning_failure = (node.fault == faults.CLEAN_FAILURE)
+        if not (fast_track or cleaning_failure):
+            manager_utils.node_power_action(task, states.POWER_OFF)
         task.driver.boot.clean_up_ramdisk(task)
         power_state_to_restore = manager_utils.power_on_node_if_needed(task)
         task.driver.network.remove_cleaning_network(task)
-        manager_utils.restore_power_state_if_needed(
-            task, power_state_to_restore)
+        if not (fast_track or cleaning_failure):
+            manager_utils.restore_power_state_if_needed(
+                task, power_state_to_restore)
 
-    @METRICS.timer('AnsibleDeploy.continue_deploy')
-    def continue_deploy(self, task):
+    @METRICS.timer('AnsibleDeploy.write_image')
+    @base.deploy_step(priority=80)
+    def write_image(self, task):
         # NOTE(pas-ha) the lock should be already upgraded in heartbeat,
         # just setting its purpose for better logging
         task.upgrade_lock(purpose='deploy')
-        task.process_event('resume')
         # NOTE(pas-ha) this method is called from heartbeat processing only,
         # so we are sure we need this particular method, not the general one
         node_address = _get_node_ip(task)
         self._ansible_deploy(task, node_address)
-        self.reboot_to_instance(task)
-
-    @METRICS.timer('AnsibleDeploy.reboot_to_instance')
-    def reboot_to_instance(self, task):
-        node = task.node
-        LOG.info('Ansible complete deploy on node %s', node.uuid)
-
-        LOG.debug('Rebooting node %s to instance', node.uuid)
+        LOG.info('Ansible complete deploy on node %s', task.node.uuid)
         manager_utils.node_set_boot_device(task, 'disk', persistent=True)
-        self.reboot_and_finish_deploy(task)
-        task.driver.boot.clean_up_ramdisk(task)
 
-        if not node.deploy_step:
-            # TODO(rloo): delete this 'if' part after deprecation period, when
-            # we expect all (out-of-tree) drivers to support deploy steps.
-            # After which we will always notify_conductor_resume_deploy().
-            task.process_event('done')
-            LOG.info('Deployment to node %s done', task.node.uuid)
-        else:
-            manager_utils.notify_conductor_resume_deploy(task)
+    @METRICS.timer('AnsibleDeploy.tear_down_agent')
+    @base.deploy_step(priority=40)
+    @task_manager.require_exclusive_lock
+    def tear_down_agent(self, task):
+        """A deploy step to tear down the agent.
 
-    @METRICS.timer('AnsibleDeploy.reboot_and_finish_deploy')
-    def reboot_and_finish_deploy(self, task):
-        wait = CONF.ansible.post_deploy_get_power_state_retry_interval * 1000
+        Shuts down the machine and removes it from the provisioning
+        network.
+
+        :param task: a TaskManager object containing the node
+        """
+        wait = CONF.ansible.post_deploy_get_power_state_retry_interval
         attempts = CONF.ansible.post_deploy_get_power_state_retries + 1
 
-        @retrying.retry(
-            stop_max_attempt_number=attempts,
-            retry_on_result=lambda state: state != states.POWER_OFF,
-            wait_fixed=wait
-        )
+        @tenacity.retry(
+            stop=tenacity.stop_after_attempt(attempts),
+            retry=(tenacity.retry_if_result(
+                lambda state: state != states.POWER_OFF)
+                | tenacity.retry_if_exception_type(Exception)),
+            wait=tenacity.wait_fixed(wait),
+            reraise=True)
         def _wait_until_powered_off(task):
             return task.driver.power.get_power_state(task)
 
@@ -656,13 +677,6 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 manager_utils.node_power_action(task, states.POWER_OFF)
-            power_state_to_restore = (
-                manager_utils.power_on_node_if_needed(task))
-            task.driver.network.remove_provisioning_network(task)
-            task.driver.network.configure_tenant_networks(task)
-            manager_utils.restore_power_state_if_needed(
-                task, power_state_to_restore)
-            manager_utils.node_power_action(task, states.POWER_ON)
         except Exception as e:
             msg = (_('Error rebooting node %(node)s after deploy. '
                      'Error: %(error)s') %
