@@ -55,7 +55,7 @@ LOG = logging.getLogger(__name__)
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 SUPPORTED_CAPABILITIES = {
-    'boot_option': ('local', 'netboot', 'ramdisk'),
+    'boot_option': ('local', 'netboot', 'ramdisk', 'kickstart'),
     'boot_mode': ('bios', 'uefi'),
     'secure_boot': ('true', 'false'),
     'trusted_boot': ('true', 'false'),
@@ -131,7 +131,8 @@ def _replace_root_uuid(path, root_uuid):
 
 def _replace_boot_line(path, boot_mode, is_whole_disk_image,
                        trusted_boot=False, iscsi_boot=False,
-                       ramdisk_boot=False, ipxe_enabled=False):
+                       ramdisk_boot=False, ipxe_enabled=False,
+                       anaconda_boot=False):
     if is_whole_disk_image:
         boot_disk_type = 'boot_whole_disk'
     elif trusted_boot:
@@ -140,6 +141,8 @@ def _replace_boot_line(path, boot_mode, is_whole_disk_image,
         boot_disk_type = 'boot_iscsi'
     elif ramdisk_boot:
         boot_disk_type = 'boot_ramdisk'
+    elif anaconda_boot:
+        boot_disk_type = 'boot_anaconda'
     else:
         boot_disk_type = 'boot_partition'
 
@@ -163,7 +166,7 @@ def _replace_disk_identifier(path, disk_identifier):
 def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
                       is_whole_disk_image, trusted_boot=False,
                       iscsi_boot=False, ramdisk_boot=False,
-                      ipxe_enabled=False):
+                      ipxe_enabled=False, anaconda_boot=False):
     """Switch a pxe config from deployment mode to service mode.
 
     :param path: path to the pxe config file in tftpboot.
@@ -178,15 +181,17 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
     :param ramdisk_boot: if the boot is to be to a ramdisk configuration.
     :param ipxe_enabled: A default False boolean value to tell the method
                          if the caller is using iPXE.
+    :param anaconda_boot: if the boot is to be to an anaconda configuration.
     """
-    if not ramdisk_boot and root_uuid_or_disk_id is not None:
+    if (not (ramdisk_boot or anaconda_boot)
+            and root_uuid_or_disk_id is not None):
         if not is_whole_disk_image:
             _replace_root_uuid(path, root_uuid_or_disk_id)
         else:
             _replace_disk_identifier(path, root_uuid_or_disk_id)
 
     _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot,
-                       iscsi_boot, ramdisk_boot, ipxe_enabled)
+                       iscsi_boot, ramdisk_boot, ipxe_enabled, anaconda_boot)
 
 
 def check_for_missing_params(info_dict, error_msg, param_prefix=''):
@@ -308,7 +313,16 @@ def agent_add_clean_params(task):
     zeroize = CONF.deploy.shred_final_overwrite_with_zeros
     info['agent_erase_devices_zeroize'] = zeroize
     erase_fallback = CONF.deploy.continue_if_disk_secure_erase_fails
+    info['agent_continue_if_secure_erase_failed'] = erase_fallback
+    # NOTE(janders) ``agent_continue_if_ata_erase_failed`` is deprecated and
+    # will be removed in the "Y" cycle. The replacement option
+    # ``agent_continue_if_secure_erase_failed`` is used to control shred
+    #  fallback for both ATA Secure Erase and NVMe Secure Erase.
+    # The ``agent_continue_if_ata_erase_failed`` line can
+    # be deleted along with this comment when support for it is fully removed.
     info['agent_continue_if_ata_erase_failed'] = erase_fallback
+    nvme_secure_erase = CONF.deploy.enable_nvme_secure_erase
+    info['agent_enable_nvme_secure_erase'] = nvme_secure_erase
     secure_erase = CONF.deploy.enable_ata_secure_erase
     info['agent_enable_ata_secure_erase'] = secure_erase
     info['disk_erasure_concurrency'] = CONF.deploy.disk_erasure_concurrency
@@ -572,11 +586,25 @@ def get_boot_option(node):
     # NOTE(TheJulia): Software raid always implies local deployment
     if is_software_raid(node):
         return 'local'
+    if is_anaconda_deploy(node):
+        return 'kickstart'
     capabilities = utils.parse_instance_info_capabilities(node)
     return capabilities.get('boot_option',
                             CONF.deploy.default_boot_option).lower()
 
 
+def is_anaconda_deploy(node):
+    """Determine if Anaconda deploy interface is in use for the deployment.
+
+    :param node: A single Node.
+    :returns: A boolean value of True when Anaconda deploy interface is in use
+              otherwise False
+    """
+    if node.deploy_interface == 'anaconda':
+        return True
+    return False
+
+
 def is_software_raid(node):
     """Determine if software raid is in use for the deployment.
 
@@ -594,6 +622,9 @@ def is_software_raid(node):
     return software_raid
 
 
+IPA_URL_PARAM_NAME = 'ipa-api-url'
+
+
 def build_agent_options(node):
     """Build the options to be passed to the agent ramdisk.
 
@@ -602,7 +633,7 @@ def build_agent_options(node):
         agent ramdisk.
     """
     agent_config_opts = {
-        'ipa-api-url': get_ironic_api_url(),
+        IPA_URL_PARAM_NAME: get_ironic_api_url(),
     }
     return agent_config_opts
 
