@@ -20,35 +20,154 @@ from oslo_log import log as logging
 from oslo_utils import importutils
 from oslo_utils import units
 
+from ironic.common import boot_modes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.common import utils
 from ironic.drivers import base
 from ironic.drivers.modules.drac import common as drac_common
+from ironic.drivers.modules import inspect_utils
 from ironic.drivers.modules.redfish import inspect as redfish_inspect
+from ironic.drivers.modules.redfish import utils as redfish_utils
 from ironic import objects
 
 drac_exceptions = importutils.try_import('dracclient.exceptions')
+sushy = importutils.try_import('sushy')
 
 LOG = logging.getLogger(__name__)
 
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
+_PXE_DEV_ENABLED_INTERFACES = [('PxeDev1EnDis', 'PxeDev1Interface'),
+                               ('PxeDev2EnDis', 'PxeDev2Interface'),
+                               ('PxeDev3EnDis', 'PxeDev3Interface'),
+                               ('PxeDev4EnDis', 'PxeDev4Interface')]
+_BIOS_ENABLED_VALUE = 'Enabled'
+
 
 class DracRedfishInspect(redfish_inspect.RedfishInspect):
-    """iDRAC Redfish interface for inspection-related actions.
+    """iDRAC Redfish interface for inspection-related actions."""
 
-    Presently, this class entirely defers to its base class, a generic,
-    vendor-independent Redfish interface. Future resolution of Dell EMC-
-    specific incompatibilities and introduction of vendor value added
-    should be implemented by this class.
-    """
-    pass
+    def inspect_hardware(self, task):
+        """Inspect hardware to get the hardware properties.
+
+        Inspects hardware to get the essential properties.
+        It fails if any of the essential properties
+        are not received from the node.
+
+        :param task: a TaskManager instance.
+        :raises: HardwareInspectionFailure if essential properties
+                 could not be retrieved successfully.
+        :returns: The resulting state of inspection.
+
+        """
+        # Ensure we create a port for every NIC port found for consistency
+        # with our WSMAN inspect behavior and to work around a bug in some
+        # versions of the firmware where the port state is not being
+        # reported correctly.
+
+        ethernet_interfaces_mac = self._get_mac_address(task)
+        inspect_utils.create_ports_if_not_exist(task, ethernet_interfaces_mac)
+        return super(DracRedfishInspect, self).inspect_hardware(task)
+
+    def _get_mac_address(self, task):
+        """Get a list of MAC addresses
+
+        :param task: a TaskManager instance.
+        :returns: Returns list of MAC addresses.
+        """
+        system = redfish_utils.get_system(task.node)
+        # Get dictionary of ethernet interfaces
+        if system.ethernet_interfaces and system.ethernet_interfaces.summary:
+            ethernet_interfaces = system.ethernet_interfaces.get_members()
+            ethernet_interfaces_mac = {
+                interface.identity: interface.mac_address
+                for interface in ethernet_interfaces}
+            return ethernet_interfaces_mac
+        else:
+            return {}
+
+    def _get_pxe_port_macs(self, task):
+        """Get a list of PXE port MAC addresses.
+
+        :param task: a TaskManager instance.
+        :returns: Returns list of PXE port MAC addresses.
+        """
+        system = redfish_utils.get_system(task.node)
+        ethernet_interfaces_mac = self._get_mac_address(task)
+        pxe_port_macs = []
+
+        if system.boot.mode == boot_modes.UEFI:
+            # When a server is in UEFI boot mode, the PXE NIC ports are
+            # stored in the PxeDevXEnDis and PxeDevXInterface BIOS
+            # settings. Get the PXE NIC ports from these settings and
+            # their MAC addresses.
+            for param, nic in _PXE_DEV_ENABLED_INTERFACES:
+                if system.bios.attributes[param] == _BIOS_ENABLED_VALUE:
+                    nic_id = system.bios.attributes[nic]
+                    # Get MAC address of the given nic_id
+                    mac_address = ethernet_interfaces_mac[nic_id]
+                    pxe_port_macs.append(mac_address)
+        elif system.boot.mode == boot_modes.LEGACY_BIOS:
+            # When a server is in BIOS boot mode, whether or not a
+            # NIC port is set to PXE boot is stored on the NIC port
+            # itself internally to the BMC. Getting this information
+            # requires using an OEM extension to export the system
+            # configuration, as the redfish standard does not specify
+            # how to get it, and Dell does not have OEM redfish calls
+            # to selectively retrieve it at this time.
+            # Get instance of Sushy OEM manager object
+
+            for manager in system.managers:
+                try:
+                    # Get instance of Sushy OEM manager object
+                    oem_manager = manager.get_oem_extension('Dell')
+                except sushy.exceptions.OEMExtensionNotFoundError as e:
+                    error_msg = (_("Search for Sushy OEM extension package "
+                                   "'sushy-oem-idrac' failed for node "
+                                   "%(node)s. Ensure it's installed. "
+                                   " Error: %(error)s") %
+                                 {'node': task.node.uuid, 'error': e})
+                    LOG.error(error_msg)
+                    raise exception.RedfishError(error=error_msg)
+
+                try:
+                    pxe_port_macs_list = oem_manager.get_pxe_port_macs_bios(
+                        ethernet_interfaces_mac)
+                    pxe_port_macs = [mac for mac in pxe_port_macs_list]
+                    return pxe_port_macs
+                except sushy.exceptions.OEMExtensionNotFoundError as e:
+                    error_msg = (_("Search for Sushy OEM extension package "
+                                   "'sushy-oem-idrac' failed for node "
+                                   " %(node)s. Ensure it is installed. "
+                                   "Error: %(error)s") %
+                                 {'node': task.node.uuid, 'error': e})
+                    LOG.debug(error_msg)
+                    continue
+                LOG.info("Get pxe port MAC addresses for  %(node)s via OEM",
+                         {'node': task.node.uuid})
+                break
+            else:
+                error_msg = (_('iDRAC Redfish Get pxe port MAC addresse '
+                               'failed for node %(node)s, because system '
+                               '%(system)s has no '
+                               'manager%(no_manager)s.') %
+                             {'node': task.node.uuid,
+                              'system': system.uuid if system.uuid else
+                              system.identity,
+                              'no_manager': '' if not system.managers else
+                              ' which could'})
+                LOG.error(error_msg)
+                raise exception.RedfishError(error=error_msg)
+        return pxe_port_macs
 
 
 class DracWSManInspect(base.InspectInterface):
 
+    _GPU_SUPPORTED_LIST = {"TU104GL [Tesla T4]",
+                           "GV100GL [Tesla V100 PCIe 16GB]"}
+
     def get_properties(self):
         """Return the properties of the interface.
 
@@ -98,9 +217,12 @@ class DracWSManInspect(base.InspectInterface):
                 properties['cpu_arch'] = 'x86_64' if cpus[0].arch64 else 'x86'
 
             bios_settings = client.list_bios_settings()
+            video_controllers = client.list_video_controllers()
             current_capabilities = node.properties.get('capabilities', '')
             new_capabilities = {
-                'boot_mode': bios_settings["BootMode"].current_value.lower()}
+                'boot_mode': bios_settings["BootMode"].current_value.lower(),
+                'pci_gpu_devices': self._calculate_gpus(video_controllers)}
+
             capabilties = utils.get_updated_capabilities(current_capabilities,
                                                          new_capabilities)
             properties['capabilities'] = capabilties
@@ -190,6 +312,23 @@ class DracWSManInspect(base.InspectInterface):
         else:
             return cpu.cores
 
+    def _calculate_gpus(self, video_controllers):
+        """Find actual GPU count.
+
+        This method reports number of NVIDIA Tesla T4 GPU devices present
+        on the server.
+
+        :param video_controllers: list of video controllers.
+
+        :returns: returns total gpu count.
+        """
+        gpu_cnt = 0
+        for video_controller in video_controllers:
+            for gpu in self._GPU_SUPPORTED_LIST:
+                if video_controller.description == gpu:
+                    gpu_cnt += 1
+        return gpu_cnt
+
     def _get_pxe_dev_nics(self, client, nics, node):
         """Get a list of pxe device interfaces.
 
