@@ -13,7 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import base64
 import functools
+import gzip
 import json
 import os
 import shutil
@@ -27,6 +29,7 @@ from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import images
 from ironic.common import swift
+from ironic.common import utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
@@ -36,55 +39,53 @@ LOG = log.getLogger(__name__)
 
 class ImageHandler(object):
 
-    _SWIFT_MAP = {
-        "redfish": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "idrac": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "ilo5": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-        "ilo": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-    }
-
     def __init__(self, driver):
-        self._driver = driver
-        self._container = self._SWIFT_MAP[driver].get("container")
-        self._timeout = self._SWIFT_MAP[driver].get("timeout")
-        self._image_subdir = self._SWIFT_MAP[driver].get("image_subdir")
-        self._file_permission = self._SWIFT_MAP[driver].get("file_permission")
-        # To get the kernel parameters
-        self.kernel_params = self._SWIFT_MAP[driver].get("kernel_params")
+        self.update_driver_config(driver)
 
-    def _is_swift_enabled(self):
-        try:
-            return self._SWIFT_MAP[self._driver].get("swift_enabled")
-        except KeyError:
-            return False
+    def update_driver_config(self, driver):
+        _SWIFT_MAP = {
+            "redfish": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "idrac": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "ilo5": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+            "ilo": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+        }
+
+        self._driver = driver
+        self.swift_enabled = _SWIFT_MAP[driver].get("swift_enabled")
+        self._container = _SWIFT_MAP[driver].get("container")
+        self._timeout = _SWIFT_MAP[driver].get("timeout")
+        self._image_subdir = _SWIFT_MAP[driver].get("image_subdir")
+        self._file_permission = _SWIFT_MAP[driver].get("file_permission")
+        # To get the kernel parameters
+        self.kernel_params = _SWIFT_MAP[driver].get("kernel_params")
 
     def unpublish_image(self, object_name):
         """Withdraw the image previously made downloadable.
@@ -95,7 +96,7 @@ class ImageHandler(object):
 
         :param object_name: name of the published file (optional)
         """
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
 
             swift_api = swift.SwiftAPI()
@@ -118,6 +119,23 @@ class ImageHandler(object):
 
             ironic_utils.unlink_without_raise(published_file)
 
+    @classmethod
+    def unpublish_image_for_node(cls, node, prefix='', suffix=''):
+        """Withdraw the image previously made downloadable.
+
+        Depending on ironic settings, removes previously published file
+        from where it has been published - Swift or local HTTP server's
+        document root.
+
+        :param node: the node for which image was published.
+        :param prefix: object name prefix.
+        :param suffix: object name suffix.
+        """
+        name = _get_name(node, prefix=prefix, suffix=suffix)
+        cls(node.driver).unpublish_image(name)
+        LOG.debug('Removed image %(name)s for node %(node)s',
+                  {'node': node.uuid, 'name': name})
+
     def _append_filename_param(self, url, filename):
         """Append 'filename=<file>' parameter to given URL.
 
@@ -159,7 +177,7 @@ class ImageHandler(object):
         :return: a URL to download published file
         """
 
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
             timeout = self._timeout
 
@@ -196,8 +214,8 @@ class ImageHandler(object):
                 shutil.copyfile(image_file, published_file)
                 os.chmod(published_file, self._file_permission)
 
-            image_url = os.path.join(
-                CONF.deploy.http_url, self._image_subdir, object_name)
+            http_url = CONF.deploy.external_http_url or CONF.deploy.http_url
+            image_url = os.path.join(http_url, self._image_subdir, object_name)
 
         image_url = self._append_filename_param(
             image_url, os.path.basename(image_file))
@@ -205,20 +223,16 @@ class ImageHandler(object):
         return image_url
 
 
-def _get_floppy_image_name(node):
-    """Returns the floppy image name for a given node.
+def _get_name(node, prefix='', suffix=''):
+    """Get an object name for a given node.
 
     :param node: the node for which image name is to be provided.
     """
-    return "image-%s" % node.uuid
-
-
-def _get_iso_image_name(node):
-    """Returns the boot iso image name for a given node.
-
-    :param node: the node for which image name is to be provided.
-    """
-    return "boot-%s.iso" % node.uuid
+    if prefix:
+        name = "%s-%s" % (prefix, node.uuid)
+    else:
+        name = node.uuid
+    return name + suffix
 
 
 def cleanup_iso_image(task):
@@ -226,10 +240,18 @@ def cleanup_iso_image(task):
 
     :param task: A task from TaskManager.
     """
-    iso_object_name = _get_iso_image_name(task.node)
-    img_handler = ImageHandler(task.node.driver)
+    ImageHandler.unpublish_image_for_node(task.node, prefix='boot',
+                                          suffix='.iso')
 
-    img_handler.unpublish_image(iso_object_name)
+
+def override_api_url(params):
+    if not CONF.deploy.external_callback_url:
+        return params
+
+    params = params or {}
+    params[deploy_utils.IPA_URL_PARAM_NAME] = \
+        CONF.deploy.external_callback_url.rstrip('/')
+    return params
 
 
 def prepare_floppy_image(task, params=None):
@@ -251,7 +273,8 @@ def prepare_floppy_image(task, params=None):
     :raises: SwiftOperationError, if any operation with Swift fails.
     :returns: image URL for the floppy image.
     """
-    object_name = _get_floppy_image_name(task.node)
+    object_name = _get_name(task.node, prefix='image')
+    params = override_api_url(params)
 
     LOG.debug("Trying to create floppy image for node "
               "%(node)s", {'node': task.node.uuid})
@@ -280,10 +303,92 @@ def cleanup_floppy_image(task):
 
     :param task: an ironic node object.
     """
-    floppy_object_name = _get_floppy_image_name(task.node)
+    ImageHandler.unpublish_image_for_node(task.node, prefix='image')
+
+
+def prepare_configdrive_image(task, content):
+    """Prepare an image with configdrive.
+
+    Decodes base64 contents and writes it into a disk image that can be
+    attached e.g. to a virtual USB device. Images stored in Swift are
+    downloaded first.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param content: Config drive as a base64-encoded string.
+    :raises: ImageCreationFailed, if it failed while creating the image.
+    :raises: SwiftOperationError, if any operation with Swift fails.
+    :returns: image URL for the image.
+    """
+    with tempfile.TemporaryFile(dir=CONF.tempdir) as comp_tmpfile_obj:
+        if '://' in content:
+            with tempfile.NamedTemporaryFile(dir=CONF.tempdir) as tmpfile2:
+                images.fetch_into(task.context, content, tmpfile2)
+                tmpfile2.flush()
+
+                if utils.file_mime_type(tmpfile2.name) == "text/plain":
+                    tmpfile2.seek(0)
+                    base64.decode(tmpfile2, comp_tmpfile_obj)
+                else:
+                    # A binary image, use it as it is.
+                    return prepare_disk_image(task, tmpfile2.name,
+                                              prefix='configdrive')
+        else:
+            comp_tmpfile_obj.write(base64.b64decode(content))
+        comp_tmpfile_obj.seek(0)
+
+        gz = gzip.GzipFile(fileobj=comp_tmpfile_obj, mode='rb')
+        with tempfile.NamedTemporaryFile(
+                dir=CONF.tempdir, suffix='.img') as image_tmpfile_obj:
+            shutil.copyfileobj(gz, image_tmpfile_obj)
+            image_tmpfile_obj.flush()
+            return prepare_disk_image(task, image_tmpfile_obj.name,
+                                      prefix='configdrive')
+
+
+def prepare_disk_image(task, content, prefix=None):
+    """Prepare an image with the given content.
+
+    If content is already an HTTP URL, return it unchanged.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param content: Content as a string with a file name or bytes with
+        contents.
+    :param prefix: Prefix to use for the object name.
+    :raises: ImageCreationFailed, if it failed while creating the image.
+    :raises: SwiftOperationError, if any operation with Swift fails.
+    :returns: image URL for the image.
+    """
+    object_name = _get_name(task.node, prefix=prefix)
+
+    LOG.debug("Creating a disk image for node %s", task.node.uuid)
 
     img_handler = ImageHandler(task.node.driver)
-    img_handler.unpublish_image(floppy_object_name)
+    if isinstance(content, str):
+        image_url = img_handler.publish_image(content, object_name)
+    else:
+        with tempfile.NamedTemporaryFile(
+                dir=CONF.tempdir, suffix='.img') as image_tmpfile_obj:
+            image_tmpfile_obj.write(content)
+            image_tmpfile_obj.flush()
+
+            image_tmpfile = image_tmpfile_obj.name
+            image_url = img_handler.publish_image(image_tmpfile, object_name)
+
+    LOG.debug("Created a disk image %(name)s for node %(node)s, "
+              "exposed as URL %(url)s", {'node': task.node.uuid,
+                                         'name': object_name,
+                                         'url': image_url})
+
+    return image_url
+
+
+def cleanup_disk_image(task, prefix=None):
+    """Deletes the image if it was created for the node.
+
+    :param task: an ironic node object.
+    :param prefix: Prefix to use for the object name.
+    """
+    ImageHandler.unpublish_image_for_node(task.node, prefix=prefix)
 
 
 def _prepare_iso_image(task, kernel_href, ramdisk_href,
@@ -342,15 +447,23 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
 
     boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
-    LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
-              "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
-              "bootloader %(bootloader_href)s and kernel params %(params)s"
-              "", {'node': task.node.uuid,
-                   'boot_mode': boot_mode,
-                   'kernel_href': kernel_href,
-                   'ramdisk_href': ramdisk_href,
-                   'bootloader_href': bootloader_href,
-                   'params': kernel_params})
+    if base_iso:
+        # TODO(dtantsur): fix this limitation eventually (see
+        # images.create_boot_iso).
+        LOG.debug('Using pre-built %(boot_mode)s ISO %(iso)s for node '
+                  '%(node)s, custom configuration will not be available',
+                  {'boot_mode': boot_mode, 'node': task.node.uuid,
+                   'iso': base_iso})
+    else:
+        LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
+                  "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
+                  "bootloader %(bootloader_href)s and kernel params %(params)s"
+                  "", {'node': task.node.uuid,
+                       'boot_mode': boot_mode,
+                       'kernel_href': kernel_href,
+                       'ramdisk_href': ramdisk_href,
+                       'bootloader_href': bootloader_href,
+                       'params': kernel_params})
 
     with tempfile.NamedTemporaryFile(
             dir=CONF.tempdir, suffix='.iso') as boot_fileobj:
@@ -366,7 +479,7 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
             base_iso=base_iso,
             inject_files=inject_files)
 
-        iso_object_name = _get_iso_image_name(task.node)
+        iso_object_name = _get_name(task.node, prefix='boot', suffix='.iso')
 
         image_url = img_handler.publish_image(
             boot_iso_tmp_file, iso_object_name)
@@ -385,7 +498,8 @@ def _find_param(param_str, param_dict):
     for param_key in param_dict:
         if param_str in param_key:
             val = param_dict.get(param_key)
-    return val
+            if val is not None:
+                return val
 
 
 _TLS_REMOTE_FILE = 'etc/ironic-python-agent/ironic.crt'
@@ -422,18 +536,22 @@ def prepare_deploy_iso(task, params, mode, d_info):
 
     kernel_str = '%s_kernel' % mode
     ramdisk_str = '%s_ramdisk' % mode
+    iso_str = '%s_iso' % mode
     bootloader_str = 'bootloader'
 
     kernel_href = _find_param(kernel_str, d_info)
     ramdisk_href = _find_param(ramdisk_str, d_info)
+    iso_href = _find_param(iso_str, d_info)
     bootloader_href = _find_param(bootloader_str, d_info)
 
+    params = override_api_url(params)
+
     # TODO(TheJulia): At some point we should support something like
     # boot_iso for the deploy interface, perhaps when we support config
     # injection.
     prepare_iso_image = functools.partial(
         _prepare_iso_image, task, kernel_href, ramdisk_href,
-        bootloader_href=bootloader_href, params=params)
+        bootloader_href=bootloader_href, params=params, base_iso=iso_href)
 
     inject_files = {}
     if CONF.agent.api_ca_file:
