@@ -168,10 +168,15 @@ def _restore_bios_config(task):
                  'as the backup data not found.', node_uuid)
         return
 
-    def _remove_bios_config(task):
+    def _remove_bios_config(task, reboot_flag=False):
         """Remove backup bios config from the node."""
         internal_info = task.node.driver_internal_info
         internal_info.pop('irmc_bios_config', None)
+        # NOTE(tiendc): If reboot flag is raised, then the BM will
+        # reboot and cause a bug if the next clean step is in-band.
+        # See https://storyboard.openstack.org/#!/story/2002731
+        if reboot_flag:
+            internal_info['cleaning_reboot'] = True
         task.node.driver_internal_info = internal_info
         task.node.save()
 
@@ -192,7 +197,7 @@ def _restore_bios_config(task):
                                            error=e)
 
     # Remove the backup data after restoring
-    _remove_bios_config(task)
+    _remove_bios_config(task, reboot_flag=True)
 
     LOG.info('BIOS config is restored successfully on node %s',
              node_uuid)
@@ -367,3 +372,35 @@ class IRMCManagement(ipmitool.IPMIManagement):
         except exception.IRMCOperationError as e:
             raise exception.NodeCleaningFailure(node=task.node.uuid,
                                                 reason=e)
+
+    def get_secure_boot_state(self, task):
+        """Get the current secure boot state for the node.
+
+        NOTE: Not all drivers support this method. Older hardware
+              may not implement that.
+
+        :param task: A task from TaskManager.
+        :raises: MissingParameterValue if a required parameter is missing
+        :raises: DriverOperationError or its derivative in case
+                 of driver runtime error.
+        :raises: UnsupportedDriverExtension if secure boot is
+                 not supported by the driver or the hardware
+        :returns: Boolean
+        """
+        return irmc_common.get_secure_boot_mode(task.node)
+
+    def set_secure_boot_state(self, task, state):
+        """Set the current secure boot state for the node.
+
+        NOTE: Not all drivers support this method. Older hardware
+              may not implement that.
+
+        :param task: A task from TaskManager.
+        :param state: A new state as a boolean.
+        :raises: MissingParameterValue if a required parameter is missing
+        :raises: DriverOperationError or its derivative in case
+                 of driver runtime error.
+        :raises: UnsupportedDriverExtension if secure boot is
+                 not supported by the driver or the hardware
+        """
+        return irmc_common.set_secure_boot_mode(task.node, state)
