@@ -234,18 +234,28 @@ class PXEBaseMixin(object):
         boot_option = deploy_utils.get_boot_option(node)
         boot_device = None
         instance_image_info = {}
-        if boot_option == "ramdisk":
+        if boot_option == "ramdisk" or boot_option == "kickstart":
             instance_image_info = pxe_utils.get_instance_image_info(
                 task, ipxe_enabled=self.ipxe_enabled)
             pxe_utils.cache_ramdisk_kernel(task, instance_image_info,
                                            ipxe_enabled=self.ipxe_enabled)
+            if 'ks_template' in instance_image_info:
+                ks_cfg = pxe_utils.validate_kickstart_template(
+                    instance_image_info['ks_template'][1]
+                )
+                pxe_utils.validate_kickstart_file(ks_cfg)
 
-        if deploy_utils.is_iscsi_boot(task) or boot_option == "ramdisk":
+        if (deploy_utils.is_iscsi_boot(task) or boot_option == "ramdisk"
+                or boot_option == "kickstart"):
             pxe_utils.prepare_instance_pxe_config(
                 task, instance_image_info,
                 iscsi_boot=deploy_utils.is_iscsi_boot(task),
                 ramdisk_boot=(boot_option == "ramdisk"),
+                anaconda_boot=(boot_option == "kickstart"),
                 ipxe_enabled=self.ipxe_enabled)
+            pxe_utils.prepare_instance_kickstart_config(
+                task, instance_image_info,
+                anaconda_boot=(boot_option == "kickstart"))
             boot_device = boot_devices.PXE
 
         elif boot_option != "local":
@@ -331,6 +341,21 @@ class PXEBaseMixin(object):
                     "iPXE boot is enabled but no HTTP URL or HTTP "
                     "root was specified."))
 
+        # NOTE(zer0c00l): When 'kickstart' boot option is used we need to store
+        # kickstart and squashfs files in http_root directory. These files
+        # will be eventually requested by anaconda installer during deployment
+        # over http(s).
+        if deploy_utils.get_boot_option(node) == 'kickstart':
+            if not CONF.deploy.http_url or not CONF.deploy.http_root:
+                raise exception.MissingParameterValue(_(
+                    "'kickstart' boot option is set on the node but no HTTP "
+                    "URL or HTTP root was specified."))
+
+            if not CONF.anaconda.default_ks_template:
+                raise exception.MissingParameterValue(_(
+                    "'kickstart' boot option is set on the node but no "
+                    "default kickstart template is specified."))
+
         # Check the trusted_boot capabilities value.
         deploy_utils.validate_capabilities(node)
         if deploy_utils.is_trusted_boot_requested(node):
@@ -390,6 +415,8 @@ class PXEBaseMixin(object):
             props = ['boot_iso']
         elif service_utils.is_glance_image(d_info['image_source']):
             props = ['kernel_id', 'ramdisk_id']
+            if deploy_utils.get_boot_option(node) == 'kickstart':
+                props.append('squashfs_id')
         else:
             props = ['kernel', 'ramdisk']
         deploy_utils.validate_image_properties(task.context, d_info, props)
