@@ -15,6 +15,7 @@
 
 from oslo_log import log
 from oslo_utils import importutils
+import tenacity
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -28,6 +29,7 @@ from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_utils
 from ironic.drivers.modules.redfish import utils as redfish_utils
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
@@ -39,10 +41,10 @@ REQUIRED_PROPERTIES = {
 }
 
 OPTIONAL_PROPERTIES = {
-    'config_via_floppy': _("Boolean value to indicate whether or not the "
-                           "driver should use virtual media Floppy device "
-                           "for passing configuration information to the "
-                           "ramdisk. Defaults to False. Optional."),
+    'config_via_removable': _("Boolean value to indicate whether or not the "
+                              "driver should use virtual media USB or floppy "
+                              "device for passing configuration information "
+                              "to the ramdisk. Defaults to False. Optional."),
     'kernel_append_params': _("Additional kernel parameters to pass down to "
                               "instance kernel. These parameters can be "
                               "consumed by the kernel or by the applications "
@@ -67,6 +69,7 @@ RESCUE_PROPERTIES = {
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
+COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(RESCUE_PROPERTIES)
 
@@ -97,27 +100,38 @@ def _parse_driver_info(node):
     d_info = node.driver_info
 
     mode = deploy_utils.rescue_or_deploy_mode(node)
-    params_to_check = KERNEL_RAMDISK_LABELS[mode]
+    iso_param = 'redfish_%s_iso' % mode
+    iso_ref = d_info.get(iso_param)
+    if iso_ref is not None:
+        deploy_info = {iso_param: iso_ref}
+    else:
+        params_to_check = KERNEL_RAMDISK_LABELS[mode]
 
-    deploy_info = {option: d_info.get(option)
-                   for option in params_to_check}
+        deploy_info = {option: d_info.get(option)
+                       for option in params_to_check}
 
-    if not any(deploy_info.values()):
-        # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
-        # from driver_info but deploy_ramdisk comes from configuration,
-        # since it's a sign of a potential operator's mistake.
-        deploy_info = {k: getattr(CONF.conductor, k)
-                       for k in params_to_check}
+        if not any(deploy_info.values()):
+            # NOTE(dtantsur): avoid situation when e.g. deploy_kernel comes
+            # from driver_info but deploy_ramdisk comes from configuration,
+            # since it's a sign of a potential operator's mistake.
+            deploy_info = {k: getattr(CONF.conductor, k)
+                           for k in params_to_check}
 
-    error_msg = _("Error validating Redfish virtual media. Some "
-                  "parameters were missing in node's driver_info")
+        error_msg = _("Error validating Redfish virtual media. Some "
+                      "parameters were missing in node's driver_info")
 
-    deploy_utils.check_for_missing_params(deploy_info, error_msg)
+        deploy_utils.check_for_missing_params(deploy_info, error_msg)
 
     deploy_info.update(
         {option: d_info.get(option, getattr(CONF.conductor, option, None))
          for option in OPTIONAL_PROPERTIES})
 
+    if (d_info.get('config_via_removable') is None
+            and d_info.get('config_via_floppy') is not None):
+        LOG.warning('The config_via_floppy driver_info option is deprecated, '
+                    'use config_via_removable for node %s', node.uuid)
+        deploy_info['config_via_removable'] = d_info['config_via_floppy']
+
     deploy_info.update(redfish_utils.parse_driver_info(node))
 
     return deploy_info
@@ -141,19 +155,33 @@ def _parse_instance_info(node):
     return deploy_info
 
 
-def _insert_vmedia(task, boot_url, boot_device):
+def _test_retry(exception):
+    if isinstance(exception, sushy.exceptions.ServerSideError):
+        # On some Dell hw, the eject media may still be in progress
+        # https://storyboard.openstack.org/#!/story/2008504
+        LOG.warning("Boot media insert failed for node %(node)s, "
+                    "will retry after 3 seconds",
+                    {'node': exception.node_uuid})
+        return True
+    return False
+
+
+@tenacity.retry(retry=tenacity.retry_if_exception(_test_retry),
+                stop=tenacity.stop_after_attempt(3),
+                wait=tenacity.wait_fixed(3),
+                reraise=True)
+def _insert_vmedia(task, managers, boot_url, boot_device):
     """Insert bootable ISO image into virtual CD or DVD
 
     :param task: A task from TaskManager.
+    :param managers: A list of System managers.
     :param boot_url: URL to a bootable ISO image
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device not in v_media.media_types:
                 continue
@@ -169,8 +197,12 @@ def _insert_vmedia(task, boot_url, boot_device):
 
                 continue
 
-            v_media.insert_media(boot_url, inserted=True,
-                                 write_protected=True)
+            try:
+                v_media.insert_media(boot_url, inserted=True,
+                                     write_protected=True)
+            except sushy.exceptions.ServerSideError as e:
+                e.node_uuid = task.node.uuid
+                raise
 
             LOG.info("Inserted boot media %(boot_url)s into "
                      "%(boot_device)s for node "
@@ -183,19 +215,18 @@ def _insert_vmedia(task, boot_url, boot_device):
         _('No suitable virtual media device found'))
 
 
-def _eject_vmedia(task, boot_device=None):
+def _eject_vmedia(task, managers, boot_device=None):
     """Eject virtual CDs and DVDs
 
     :param task: A task from TaskManager.
+    :param managers: A list of System managers.
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
         `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
         eject everything (default).
     :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
         found on the node.
     """
-    system = redfish_utils.get_system(task.node)
-
-    for manager in system.managers:
+    for manager in managers:
         for v_media in manager.virtual_media.get_members():
             if boot_device and boot_device not in v_media.media_types:
                 continue
@@ -212,19 +243,38 @@ def _eject_vmedia(task, boot_device=None):
                           'boot_device': v_media.name})
 
 
-def _has_vmedia_device(task, boot_device):
-    """Indicate if device exists at any of the managers
+def eject_vmedia(task, boot_device=None):
+    """Eject virtual CDs and DVDs
 
     :param task: A task from TaskManager.
     :param boot_device: sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
-        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`.
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY` or `None` to
+        eject everything (default).
+    :raises: InvalidParameterValue, if no suitable virtual CD or DVD is
+        found on the node.
     """
     system = redfish_utils.get_system(task.node)
+    _eject_vmedia(task, system.managers, boot_device=boot_device)
 
-    for manager in system.managers:
-        for v_media in manager.virtual_media.get_members():
-            if boot_device in v_media.media_types:
-                return True
+
+def _has_vmedia_device(managers, boot_device):
+    """Indicate if device exists at any of the managers
+
+    :param managers: A list of System managers.
+    :param boot_device: One or more sushy boot device e.g. `VIRTUAL_MEDIA_CD`,
+        `VIRTUAL_MEDIA_DVD` or `VIRTUAL_MEDIA_FLOPPY`. Several devices are
+        checked in the given order.
+    :return: The device that could be found or False.
+    """
+    if isinstance(boot_device, str):
+        boot_device = [boot_device]
+
+    for dev in boot_device:
+        for manager in managers:
+            for v_media in manager.virtual_media.get_members():
+                if dev in v_media.media_types:
+                    return dev
+    return False
 
 
 def _parse_deploy_info(node):
@@ -279,7 +329,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
     `[instance_info]image_source` node property.
     """
 
-    capabilities = ['iscsi_volume_boot', 'ramdisk_boot']
+    capabilities = ['iscsi_volume_boot', 'ramdisk_boot',
+                    'ramdisk_boot_configdrive']
 
     def __init__(self):
         """Initialize the Redfish virtual media boot interface.
@@ -342,6 +393,19 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
         deploy_utils.validate_image_properties(task.context, d_info, props)
 
+    def _validate_vendor(self, task):
+        vendor = task.node.properties.get('vendor')
+        if not vendor:
+            return
+
+        if 'Dell' in vendor.split():
+            raise exception.InvalidParameterValue(
+                _("The %(iface)s boot interface is not suitable for node "
+                  "%(node)s with vendor %(vendor)s, use "
+                  "idrac-redfish-virtual-media instead")
+                % {'iface': task.node.boot_interface,
+                   'node': task.node.uuid, 'vendor': vendor})
+
     def validate(self, task):
         """Validate the deployment information for the task's node.
 
@@ -353,6 +417,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :raises: InvalidParameterValue on malformed parameter(s)
         :raises: MissingParameterValue on missing parameter(s)
         """
+        self._validate_vendor(task)
         self._validate_driver_info(task)
 
         if task.driver.storage.should_write_image(task):
@@ -412,7 +477,7 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
         d_info = _parse_driver_info(node)
 
-        config_via_floppy = d_info.get('config_via_floppy')
+        config_via_removable = d_info.get('config_via_removable')
 
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
         if deploy_nic_mac is not None:
@@ -420,35 +485,44 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         if CONF.debug and 'ipa-debug' not in ramdisk_params:
             ramdisk_params['ipa-debug'] = '1'
 
-        if config_via_floppy:
+        managers = redfish_utils.get_system(task.node).managers
 
-            if _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY):
-                # NOTE (etingof): IPA will read the diskette only if
-                # we tell it to
-                ramdisk_params['boot_method'] = 'vmedia'
+        # NOTE(TheJulia): This is a mandatory setting for virtual media
+        # based deployment operations.
+        ramdisk_params['boot_method'] = 'vmedia'
 
+        if config_via_removable:
+
+            removable = _has_vmedia_device(
+                managers,
+                # Prefer USB devices since floppies are outdated
+                [sushy.VIRTUAL_MEDIA_USBSTICK, sushy.VIRTUAL_MEDIA_FLOPPY])
+            if removable:
                 floppy_ref = image_utils.prepare_floppy_image(
                     task, params=ramdisk_params)
 
-                _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-                _insert_vmedia(
-                    task, floppy_ref, sushy.VIRTUAL_MEDIA_FLOPPY)
+                _eject_vmedia(task, managers, removable)
+                _insert_vmedia(task, managers, floppy_ref, removable)
 
-                LOG.debug('Inserted virtual floppy with configuration for '
-                          'node %(node)s', {'node': task.node.uuid})
+                LOG.info('Inserted virtual %(type)s device with configuration'
+                         ' for node %(node)s',
+                         {'node': task.node.uuid, 'type': removable})
 
             else:
-                LOG.warning('Config via floppy is requested, but '
-                            'Floppy drive is not available on node '
-                            '%(node)s', {'node': task.node.uuid})
+                LOG.warning('Config via a removable device is requested, but '
+                            'virtual USB and floppy devices are not '
+                            'available on node %(node)s',
+                            {'node': task.node.uuid})
 
         mode = deploy_utils.rescue_or_deploy_mode(node)
 
         iso_ref = image_utils.prepare_deploy_iso(task, ramdisk_params,
                                                  mode, d_info)
 
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        del managers
 
         boot_mode_utils.sync_boot_mode(task)
 
@@ -467,21 +541,9 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         :param task: A task from TaskManager.
         :returns: None
         """
-        d_info = _parse_driver_info(task.node)
-
-        config_via_floppy = d_info.get('config_via_floppy')
-
         LOG.debug("Cleaning up deploy boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        image_utils.cleanup_iso_image(task)
-
-        if (config_via_floppy
-                and _has_vmedia_device(task, sushy.VIRTUAL_MEDIA_FLOPPY)):
-            _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-            image_utils.cleanup_floppy_image(task)
+        self._eject_all(task)
 
     def prepare_instance(self, task):
         """Prepares the boot of instance over virtual media.
@@ -505,8 +567,12 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         node = task.node
 
+        self._eject_all(task)
+
+        boot_mode_utils.sync_boot_mode(task)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
+
         boot_option = deploy_utils.get_boot_option(node)
-        self.clean_up_instance(task)
         iwdi = node.driver_internal_info.get('is_whole_disk_image')
         if boot_option == "local" or iwdi:
             self._set_boot_device(task, boot_devices.DISK, persistent=True)
@@ -531,12 +597,26 @@ class RedfishVirtualMediaBoot(base.BootInterface):
 
             params.update(root_uuid=root_uuid)
 
-        deploy_info = _parse_deploy_info(node)
-        iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        _insert_vmedia(task, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+        managers = redfish_utils.get_system(task.node).managers
 
-        boot_mode_utils.sync_boot_mode(task)
+        deploy_info = _parse_deploy_info(node)
+        configdrive = node.instance_info.get('configdrive')
+        iso_ref = image_utils.prepare_boot_iso(task, deploy_info, **params)
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        _insert_vmedia(task, managers, iso_ref, sushy.VIRTUAL_MEDIA_CD)
+
+        if configdrive and boot_option == 'ramdisk':
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            cd_ref = image_utils.prepare_configdrive_image(task, configdrive)
+            try:
+                _insert_vmedia(task, managers, cd_ref,
+                               sushy.VIRTUAL_MEDIA_USBSTICK)
+            except exception.InvalidParameterValue:
+                raise exception.InstanceDeployFailure(
+                    _('Cannot attach configdrive for node %s: no suitable '
+                      'virtual USB slot has been found') % node.uuid)
+
+        del managers
 
         self._set_boot_device(task, boot_devices.CDROM, persistent=True)
 
@@ -544,6 +624,28 @@ class RedfishVirtualMediaBoot(base.BootInterface):
                   "%(device)s", {'node': task.node.uuid,
                                  'device': boot_devices.CDROM})
 
+    def _eject_all(self, task):
+        managers = redfish_utils.get_system(task.node).managers
+
+        _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_CD)
+        config_via_removable = (
+            task.node.driver_info.get('config_via_removable')
+            or task.node.driver_info.get('config_via_floppy')
+        )
+        if config_via_removable:
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_FLOPPY)
+
+            image_utils.cleanup_floppy_image(task)
+
+        boot_option = deploy_utils.get_boot_option(task.node)
+        if (boot_option == 'ramdisk'
+                and task.node.instance_info.get('configdrive')):
+            _eject_vmedia(task, managers, sushy.VIRTUAL_MEDIA_USBSTICK)
+            image_utils.cleanup_disk_image(task, prefix='configdrive')
+
+        image_utils.cleanup_iso_image(task)
+
     def clean_up_instance(self, task):
         """Cleans up the boot of instance.
 
@@ -555,14 +657,8 @@ class RedfishVirtualMediaBoot(base.BootInterface):
         """
         LOG.debug("Cleaning up instance boot for "
                   "%(node)s", {'node': task.node.uuid})
-
-        _eject_vmedia(task, sushy.VIRTUAL_MEDIA_CD)
-        d_info = task.node.driver_info
-        config_via_floppy = d_info.get('config_via_floppy')
-        if config_via_floppy:
-            _eject_vmedia(task, sushy.VIRTUAL_MEDIA_FLOPPY)
-
-        image_utils.cleanup_iso_image(task)
+        self._eject_all(task)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
     @classmethod
     def _set_boot_device(cls, task, device, persistent=False):
