@@ -272,6 +272,9 @@ def get_ramdisk_logs_file_name(node, label=None):
     """
     timestamp = timeutils.utcnow().strftime('%Y-%m-%d-%H-%M-%S')
     file_name_fields = [node.uuid]
+    if node.name:
+        file_name_fields.append(node.name)
+
     if node.instance_uuid:
         file_name_fields.append(node.instance_uuid)
 
@@ -323,7 +326,7 @@ def store_ramdisk_logs(node, logs, label=None):
                 f.name, object_headers=object_headers)
 
 
-def collect_ramdisk_logs(node):
+def collect_ramdisk_logs(node, label=None):
     """Collect and store the system logs from the IPA ramdisk.
 
     Collect and store the system logs from the IPA ramdisk. This method
@@ -331,8 +334,11 @@ def collect_ramdisk_logs(node):
     according to the configured storage backend.
 
     :param node: A node object.
-
+    :param label: A string to label the log file such as a clean step name.
     """
+    if CONF.agent.deploy_logs_collect == 'never':
+        return
+
     client = agent_client.AgentClient()
     try:
         result = client.collect_system_logs(node)
@@ -350,7 +356,8 @@ def collect_ramdisk_logs(node):
         return
 
     try:
-        store_ramdisk_logs(node, result['command_result']['system_logs'])
+        store_ramdisk_logs(node, result['command_result']['system_logs'],
+                           label=label)
     except exception.SwiftOperationError as e:
         LOG.error('Failed to store the logs from the node %(node)s '
                   'deployment in Swift. Error: %(error)s',
@@ -363,3 +370,17 @@ def collect_ramdisk_logs(node):
         LOG.exception('Unknown error when storing logs from the node '
                       '%(node)s deployment. Error: %(error)s',
                       {'node': node.uuid, 'error': e})
+
+
+OPTIONAL_PROPERTIES = {
+    'force_persistent_boot_device': _("Controls the persistency of boot order "
+                                      "changes. 'Always' will make all "
+                                      "changes persistent, 'Default' will "
+                                      "make all but the final one upon "
+                                      "instance deployment non-persistent, "
+                                      "and 'Never' will make no persistent "
+                                      "changes at all. The old values 'True' "
+                                      "and 'False' are still supported but "
+                                      "deprecated in favor of the new ones."
+                                      "Defaults to 'Default'. Optional."),
+}
