@@ -16,8 +16,8 @@
 """Tests for manipulating Conductors via the DB API"""
 
 import datetime
+from unittest import mock
 
-import mock
 import oslo_db
 from oslo_db import exception as db_exc
 from oslo_db import sqlalchemy
@@ -48,18 +48,25 @@ class DbConductorTestCase(base.DbTestCase):
         c = utils.get_test_conductor(**kwargs)
         cdr = self.dbapi.register_conductor(c)
         for ht in hardware_types:
-            self.dbapi.register_conductor_hardware_interfaces(cdr.id, ht,
-                                                              'power',
-                                                              ['ipmi', 'fake'],
-                                                              'ipmi')
+            self.dbapi.register_conductor_hardware_interfaces(
+                cdr.id,
+                [{'hardware_type': ht, 'interface_type': 'power',
+                  'interface_name': 'ipmi', 'default': True},
+                 {'hardware_type': ht, 'interface_type': 'power',
+                  'interface_name': 'fake', 'default': False}]
+            )
         return cdr
 
     def test_register_conductor_hardware_interfaces(self):
         c = self._create_test_cdr()
         interfaces = ['direct', 'iscsi']
-        self.dbapi.register_conductor_hardware_interfaces(c.id, 'generic',
-                                                          'deploy', interfaces,
-                                                          'iscsi')
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id,
+            [{'hardware_type': 'generic', 'interface_type': 'deploy',
+              'interface_name': interfaces[0], 'default': False},
+             {'hardware_type': 'generic', 'interface_type': 'deploy',
+              'interface_name': interfaces[1], 'default': True}]
+        )
         ifaces = self.dbapi.list_conductor_hardware_interfaces(c.id)
         ci1, ci2 = ifaces
         self.assertEqual(2, len(ifaces))
@@ -74,10 +81,13 @@ class DbConductorTestCase(base.DbTestCase):
 
     def test_register_conductor_hardware_interfaces_duplicate(self):
         c = self._create_test_cdr()
-        interfaces = ['direct', 'iscsi']
-        self.dbapi.register_conductor_hardware_interfaces(c.id, 'generic',
-                                                          'deploy', interfaces,
-                                                          'iscsi')
+        interfaces = [
+            {'hardware_type': 'generic', 'interface_type': 'deploy',
+             'interface_name': 'direct', 'default': False},
+            {'hardware_type': 'generic', 'interface_type': 'deploy',
+             'interface_name': 'iscsi', 'default': True}
+        ]
+        self.dbapi.register_conductor_hardware_interfaces(c.id, interfaces)
         ifaces = self.dbapi.list_conductor_hardware_interfaces(c.id)
         ci1, ci2 = ifaces
         self.assertEqual(2, len(ifaces))
@@ -86,14 +96,18 @@ class DbConductorTestCase(base.DbTestCase):
         self.assertRaises(
             exception.ConductorHardwareInterfacesAlreadyRegistered,
             self.dbapi.register_conductor_hardware_interfaces,
-            c.id, 'generic', 'deploy', interfaces, 'iscsi')
+            c.id, interfaces)
 
     def test_unregister_conductor_hardware_interfaces(self):
         c = self._create_test_cdr()
         interfaces = ['direct', 'iscsi']
-        self.dbapi.register_conductor_hardware_interfaces(c.id, 'generic',
-                                                          'deploy', interfaces,
-                                                          'iscsi')
+        self.dbapi.register_conductor_hardware_interfaces(
+            c.id,
+            [{'hardware_type': 'generic', 'interface_type': 'deploy',
+              'interface_name': interfaces[0], 'default': False},
+             {'hardware_type': 'generic', 'interface_type': 'deploy',
+              'interface_name': interfaces[1], 'default': True}]
+        )
         self.dbapi.unregister_conductor_hardware_interfaces(c.id)
 
         ifaces = self.dbapi.list_conductor_hardware_interfaces(c.id)
@@ -179,13 +193,16 @@ class DbConductorTestCase(base.DbTestCase):
         node1 = self.dbapi.create_node({'reservation': 'hostname1'})
         node2 = self.dbapi.create_node({'reservation': 'hostname2'})
         node3 = self.dbapi.create_node({'reservation': None})
+        node4 = self.dbapi.create_node({'reservation': 'hostName1'})
         self.dbapi.clear_node_reservations_for_conductor('hostname1')
         node1 = self.dbapi.get_node_by_id(node1.id)
         node2 = self.dbapi.get_node_by_id(node2.id)
         node3 = self.dbapi.get_node_by_id(node3.id)
+        node4 = self.dbapi.get_node_by_id(node4.id)
         self.assertIsNone(node1.reservation)
         self.assertEqual('hostname2', node2.reservation)
         self.assertIsNone(node3.reservation)
+        self.assertIsNone(node4.reservation)
 
     def test_clear_node_target_power_state(self):
         node1 = self.dbapi.create_node({'reservation': 'hostname1',
@@ -194,16 +211,21 @@ class DbConductorTestCase(base.DbTestCase):
                                         'target_power_state': 'power on'})
         node3 = self.dbapi.create_node({'reservation': None,
                                         'target_power_state': 'power on'})
+        node4 = self.dbapi.create_node({'reservation': 'hostName1',
+                                        'target_power_state': 'power on'})
         self.dbapi.clear_node_target_power_state('hostname1')
         node1 = self.dbapi.get_node_by_id(node1.id)
         node2 = self.dbapi.get_node_by_id(node2.id)
         node3 = self.dbapi.get_node_by_id(node3.id)
+        node4 = self.dbapi.get_node_by_id(node4.id)
         self.assertIsNone(node1.target_power_state)
         self.assertIn('power operation was aborted', node1.last_error)
         self.assertEqual('power on', node2.target_power_state)
         self.assertIsNone(node2.last_error)
         self.assertEqual('power on', node3.target_power_state)
         self.assertIsNone(node3.last_error)
+        self.assertIsNone(node4.target_power_state)
+        self.assertIn('power operation was aborted', node4.last_error)
 
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
     def test_get_active_hardware_type_dict_one_host_no_ht(self, mock_utcnow):
