@@ -15,8 +15,7 @@
 
 """Test class for Management Interface used by iLO modules."""
 
-import mock
-import six
+from unittest import mock
 
 from ironic.common import exception
 from ironic.common import states
@@ -365,7 +364,7 @@ class TestInspectPrivateMethods(test_common.BaseIloTest):
                                    self.node,
                                    ilo_mock)
         self.assertEqual(
-            six.text_type(result),
+            str(result),
             ("Failed to inspect hardware. Reason: Server didn't return the "
              "key(s): cpu_arch"))
 
@@ -467,3 +466,15 @@ class TestInspectPrivateMethods(test_common.BaseIloTest):
             expected.update({key: 'true'})
         cap = ilo_inspect._create_supported_capabilities_dict(capabilities)
         self.assertEqual(expected, cap)
+
+    def test___create_supported_capabilities_dict_gpu_capabilities(self):
+        capabilities = {'gpu_Nvidia_count': 1, 'gpu_Nvidia_Tesla_M10_count': 1,
+                        'gpu_Nvidia_Tesla_M10': True}
+        expected = {}
+        expected.update(capabilities)
+        for key in ilo_inspect.CAPABILITIES_KEYS:
+            capabilities.update({key: 'true'})
+            expected.update({key: 'true'})
+        capabilities.update({'unknown_property': 'true'})
+        cap = ilo_inspect._create_supported_capabilities_dict(capabilities)
+        self.assertEqual(expected, cap)
