@@ -53,7 +53,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         self.port = utils.create_test_port(
             self.context, node_id=self.node.id,
             address='52:54:00:cf:2d:32',
-            extra={'vif_port_id': uuidutils.generate_uuid()})
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
         self.neutron_port = stubs.FakeNeutronPort(
             id='132f871f-eaec-4fed-9475-0d54465e0f00',
             mac_address='52:54:00:cf:2d:32')
@@ -425,7 +425,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.context, node_id=self.node.id,
             address='52:54:00:cf:2d:33',
             uuid=uuidutils.generate_uuid(),
-            extra={'vif_port_id': uuidutils.generate_uuid()})
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
         neutron_other_port = {'id': uuidutils.generate_uuid(),
                               'mac_address': '52:54:00:cf:2d:33'}
         add_ports_mock.return_value = {
@@ -456,7 +456,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.context, node_id=self.node.id,
             address='52:54:00:cf:2d:33',
             uuid=uuidutils.generate_uuid(),
-            extra={'vif_port_id': uuidutils.generate_uuid()})
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
         neutron_other_port = {'id': uuidutils.generate_uuid(),
                               'mac_address': '52:54:00:cf:2d:33'}
         add_ports_mock.return_value = {
@@ -514,7 +514,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             self.context, node_id=self.node.id,
             address='52:54:00:cf:2d:33',
             uuid=uuidutils.generate_uuid(),
-            extra={'vif_port_id': uuidutils.generate_uuid()})
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
         other_port.internal_info = {'rescuing_vif_port_id': 'vif-port-id'}
         other_port.save()
         with task_manager.acquire(self.context, self.node.id) as task:
@@ -533,7 +533,9 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.unconfigure_tenant_networks(task)
             mock_unbind_port.assert_called_once_with(
-                self.port.extra['vif_port_id'], context=task.context)
+                self.port.internal_info['tenant_vif_port_id'],
+                context=task.context,
+                reset_mac=True)
 
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     @mock.patch.object(neutron_common, 'wait_for_host_agent', autospec=True)
@@ -550,9 +552,39 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.unconfigure_tenant_networks(task)
             mock_unbind_port.assert_called_once_with(
-                self.port.extra['vif_port_id'], context=task.context)
+                self.port.internal_info['tenant_vif_port_id'],
+                context=task.context,
+                reset_mac=True)
             wait_agent_mock.assert_called_once_with(nclient, 'hostname')
 
+    @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
+    def test_unconfigure_tenant_networks_portgroup_1(self, mock_unbind_port):
+        pg = utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address='ff:54:00:cf:2d:32',
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
+        with task_manager.acquire(self.context, self.node.id) as task:
+            self.interface.unconfigure_tenant_networks(task)
+            mock_unbind_port.assert_has_calls([
+                mock.call(self.port.internal_info['tenant_vif_port_id'],
+                          context=task.context,
+                          reset_mac=True),
+                mock.call(pg.internal_info['tenant_vif_port_id'],
+                          context=task.context, reset_mac=True)])
+
+    @mock.patch.object(neutron_common, 'unbind_neutron_port', autospec=True)
+    def test_unconfigure_tenant_networks_portgroup_2(self, mock_unbind_port):
+        pg = utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address=None,
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
+        with task_manager.acquire(self.context, self.node.id) as task:
+            self.interface.unconfigure_tenant_networks(task)
+            mock_unbind_port.assert_has_calls([
+                mock.call(self.port.internal_info['tenant_vif_port_id'],
+                          context=task.context,
+                          reset_mac=True),
+                mock.call(pg.internal_info['tenant_vif_port_id'],
+                          context=task.context, reset_mac=False)])
+
     def test_configure_tenant_networks_no_ports_for_node(self):
         n = utils.create_test_node(self.context, network_interface='neutron',
                                    uuid=uuidutils.generate_uuid())
@@ -564,7 +596,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     @mock.patch.object(neutron, 'LOG', autospec=True)
     def test_configure_tenant_networks_no_vif_id(self, log_mock, client_mock):
-        self.port.extra = {}
+        self.port.internal_info = {}
         self.port.save()
         upd_mock = mock.Mock()
         client_mock.return_value.update_port = upd_mock
@@ -596,9 +628,10 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.configure_tenant_networks(task)
             client_mock.assert_called_once_with(context=task.context)
-        update_mock.assert_called_once_with(self.context,
-                                            self.port.extra['vif_port_id'],
-                                            expected_attrs)
+        update_mock.assert_called_once_with(
+            self.context,
+            self.port.internal_info['tenant_vif_port_id'],
+            expected_attrs)
 
     @mock.patch.object(neutron_common, 'wait_for_host_agent', autospec=True)
     @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
@@ -619,16 +652,18 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
     @mock.patch.object(neutron_common, 'get_client', autospec=True)
     def _test_configure_tenant_networks(self, client_mock, update_mock,
                                         wait_agent_mock,
-                                        is_client_id=False,
-                                        vif_int_info=False):
-        if vif_int_info:
-            kwargs = {'internal_info': {
+                                        is_client_id=False):
+        # NOTE(TheJulia): Until we have a replacement for infiniband client-id
+        # storage, extra has to stay put. On a plus side, this would be
+        # pointless/difficult to abuse other than just break dhcp for the node.
+        extra = {}
+        tenant_vif = self.port.internal_info['tenant_vif_port_id']
+        kwargs = {
+            'internal_info': {
                 'tenant_vif_port_id': uuidutils.generate_uuid()}}
-            self.port.internal_info = {
-                'tenant_vif_port_id': self.port.extra['vif_port_id']}
-            self.port.extra = {}
-        else:
-            kwargs = {'extra': {'vif_port_id': uuidutils.generate_uuid()}}
+        self.port.internal_info = {
+            'tenant_vif_port_id': tenant_vif}
+        self.port.extra = {}
         second_port = utils.create_test_port(
             self.context, node_id=self.node.id, address='52:54:00:cf:2d:33',
             uuid=uuidutils.generate_uuid(),
@@ -641,7 +676,6 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             client_ids = (CLIENT_ID1, CLIENT_ID2)
             ports = (self.port, second_port)
             for port, client_id in zip(ports, client_ids):
-                extra = port.extra
                 extra['client-id'] = client_id
                 port.extra = extra
                 port.save()
@@ -666,31 +700,19 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
         with task_manager.acquire(self.context, self.node.id) as task:
             self.interface.configure_tenant_networks(task)
             client_mock.assert_called_once_with(context=task.context)
-        if vif_int_info:
-            portid1 = self.port.internal_info['tenant_vif_port_id']
-            portid2 = second_port.internal_info['tenant_vif_port_id']
-        else:
-            portid1 = self.port.extra['vif_port_id']
-            portid2 = second_port.extra['vif_port_id']
+        portid1 = self.port.internal_info['tenant_vif_port_id']
+        portid2 = second_port.internal_info['tenant_vif_port_id']
         update_mock.assert_has_calls(
             [mock.call(self.context, portid1, port1_attrs),
              mock.call(self.context, portid2, port2_attrs)],
             any_order=True
         )
 
-    def test_configure_tenant_networks_vif_extra(self):
+    def test_configure_tenant_networks(self):
         self.node.instance_uuid = uuidutils.generate_uuid()
         self.node.save()
         self._test_configure_tenant_networks()
 
-    def test_configure_tenant_networks_vif_int_info(self):
-        self.node.instance_uuid = uuidutils.generate_uuid()
-        self.node.save()
-        self._test_configure_tenant_networks(vif_int_info=True)
-
-    def test_configure_tenant_networks_no_instance_uuid(self):
-        self._test_configure_tenant_networks()
-
     def test_configure_tenant_networks_with_client_id(self):
         self.node.instance_uuid = uuidutils.generate_uuid()
         self.node.save()
@@ -707,7 +729,7 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             port_data_mock):
         pg = utils.create_test_portgroup(
             self.context, node_id=self.node.id, address='ff:54:00:cf:2d:32',
-            extra={'vif_port_id': uuidutils.generate_uuid()})
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
         port1 = utils.create_test_port(
             self.context, node_id=self.node.id, address='ff:54:00:cf:2d:33',
             uuid=uuidutils.generate_uuid(),
@@ -749,9 +771,71 @@ class NeutronInterfaceTestCase(db_base.DbTestCase):
             client_mock.assert_called_once_with(context=task.context)
             glgi_mock.assert_called_once_with(task, pg)
         update_mock.assert_has_calls(
-            [mock.call(self.context, self.port.extra['vif_port_id'],
+            [mock.call(self.context,
+                       self.port.internal_info['tenant_vif_port_id'],
                        call1_attrs),
-             mock.call(self.context, pg.extra['vif_port_id'],
+             mock.call(self.context,
+                       pg.internal_info['tenant_vif_port_id'],
+                       call2_attrs)]
+        )
+
+    @mock.patch.object(neutron_common, 'get_neutron_port_data', autospec=True)
+    @mock.patch.object(neutron_common, 'wait_for_host_agent', autospec=True)
+    @mock.patch.object(neutron_common, 'update_neutron_port', autospec=True)
+    @mock.patch.object(neutron_common, 'get_client', autospec=True)
+    @mock.patch.object(neutron_common, 'get_local_group_information',
+                       autospec=True)
+    def test_configure_tenant_networks_with_portgroups_no_address(
+            self, glgi_mock, client_mock, update_mock, wait_agent_mock,
+            port_data_mock):
+        pg = utils.create_test_portgroup(
+            self.context, node_id=self.node.id, address=None,
+            internal_info={'tenant_vif_port_id': uuidutils.generate_uuid()})
+        port1 = utils.create_test_port(
+            self.context, node_id=self.node.id, address='ff:54:00:cf:2d:33',
+            uuid=uuidutils.generate_uuid(),
+            portgroup_id=pg.id,
+            local_link_connection={'switch_id': '0a:1b:2c:3d:4e:ff',
+                                   'port_id': 'Ethernet1/1',
+                                   'switch_info': 'switch2'}
+        )
+        port2 = utils.create_test_port(
+            self.context, node_id=self.node.id, address='ff:54:00:cf:2d:34',
+            uuid=uuidutils.generate_uuid(),
+            portgroup_id=pg.id,
+            local_link_connection={'switch_id': '0a:1b:2c:3d:4e:ff',
+                                   'port_id': 'Ethernet1/2',
+                                   'switch_info': 'switch2'}
+        )
+        local_group_info = {'a': 'b'}
+        glgi_mock.return_value = local_group_info
+        expected_attrs = {'binding:vnic_type': 'baremetal',
+                          'binding:host_id': self.node.uuid}
+        call1_attrs = copy.deepcopy(expected_attrs)
+        call1_attrs['binding:profile'] = {
+            'local_link_information': [self.port.local_link_connection]
+        }
+        call1_attrs['mac_address'] = '52:54:00:cf:2d:32'
+        call2_attrs = copy.deepcopy(expected_attrs)
+        call2_attrs['binding:profile'] = {
+            'local_link_information': [port1.local_link_connection,
+                                       port2.local_link_connection],
+            'local_group_information': local_group_info
+        }
+        with task_manager.acquire(self.context, self.node.id) as task:
+            # Override task.portgroups here, to have ability to check
+            # that mocked get_local_group_information was called with
+            # this portgroup object.
+            task.portgroups = [pg]
+            self.interface.configure_tenant_networks(task)
+            client_mock.assert_called_once_with(context=task.context)
+            glgi_mock.assert_called_once_with(task, pg)
+        update_mock.assert_has_calls(
+            [mock.call(self.context,
+                       self.port.internal_info['tenant_vif_port_id'],
+                       call1_attrs),
+             mock.call(self.context,
+                       pg.internal_info['tenant_vif_port_id'],
                        call2_attrs)]
         )
 
