@@ -17,6 +17,7 @@ import datetime
 from unittest import mock
 
 from oslo_utils import importutils
+from oslo_utils import units
 
 from ironic.common import boot_devices
 from ironic.common import boot_modes
@@ -56,6 +57,28 @@ class RedfishManagementTestCase(db_base.DbTestCase):
         self.chassis_uuid = 'XXX-YYY-ZZZ'
         self.drive_uuid = 'ZZZ-YYY-XXX'
 
+    def init_system_mock(self, system_mock, **properties):
+
+        system_mock.reset()
+
+        system_mock.boot.mode = 'uefi'
+
+        system_mock.memory_summary.size_gib = 2
+
+        system_mock.processors.summary = '8', 'MIPS'
+
+        system_mock.simple_storage.disks_sizes_bytes = (
+            1 * units.Gi, units.Gi * 3, units.Gi * 5)
+        system_mock.storage.volumes_sizes_bytes = (
+            2 * units.Gi, units.Gi * 4, units.Gi * 6)
+
+        system_mock.ethernet_interfaces.summary = {
+            '00:11:22:33:44:55': sushy.STATE_ENABLED,
+            '66:77:88:99:AA:BB': sushy.STATE_DISABLED,
+        }
+
+        return system_mock
+
     @mock.patch.object(redfish_mgmt, 'sushy', None)
     def test_loading_error(self):
         self.assertRaisesRegex(
@@ -104,8 +127,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                 # Asserts
                 fake_system.set_system_boot_options.assert_has_calls(
                     [mock.call(expected,
-                               enabled=sushy.BOOT_SOURCE_ENABLED_ONCE),
-                     mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                               enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)])
                 mock_get_system.assert_called_with(task.node)
                 self.assertNotIn('redfish_boot_device',
                                  task.node.driver_internal_info)
@@ -131,8 +153,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
                 fake_system.set_system_boot_options.assert_has_calls(
                     [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
-                               enabled=expected),
-                     mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                               enabled=expected)])
                 mock_get_system.assert_called_with(task.node)
                 self.assertNotIn('redfish_boot_device',
                                  task.node.driver_internal_info)
@@ -159,8 +180,7 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                     task, boot_devices.PXE, persistent=target)
 
                 fake_system.set_system_boot_options.assert_has_calls(
-                    [mock.call(sushy.BOOT_SOURCE_TARGET_PXE, enabled=None),
-                     mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                    [mock.call(sushy.BOOT_SOURCE_TARGET_PXE, enabled=None)])
                 mock_get_system.assert_called_with(task.node)
 
                 # Reset mocks
@@ -266,11 +286,15 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                                       shared=False) as task:
                 task.driver.management.set_boot_device(
                     task, boot_devices.PXE, persistent=True)
-
-                fake_system.set_system_boot_options.assert_has_calls(
-                    [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
-                               enabled=expected),
-                     mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                if vendor == 'SuperMicro':
+                    fake_system.set_system_boot_options.assert_has_calls(
+                        [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
+                                   enabled=expected),
+                         mock.call(mode=sushy.BOOT_SOURCE_MODE_BIOS)])
+                else:
+                    fake_system.set_system_boot_options.assert_has_calls(
+                        [mock.call(sushy.BOOT_SOURCE_TARGET_PXE,
+                                   enabled=expected)])
 
                 # Reset mocks
                 fake_system.set_system_boot_options.reset_mock()
@@ -350,6 +374,12 @@ class RedfishManagementTestCase(db_base.DbTestCase):
 
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_set_boot_mode(self, mock_get_system):
+        boot_attribute = {
+            'target': sushy.BOOT_SOURCE_TARGET_PXE,
+            'enabled': sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
+            'mode': sushy.BOOT_SOURCE_MODE_BIOS,
+        }
+        fake_system = mock.Mock(boot=boot_attribute)
         fake_system = mock.Mock()
         mock_get_system.return_value = fake_system
         with task_manager.acquire(self.context, self.node.uuid,
@@ -374,7 +404,12 @@ class RedfishManagementTestCase(db_base.DbTestCase):
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_set_boot_mode_fail(self, mock_get_system, mock_sushy):
-        fake_system = mock.Mock()
+        boot_attribute = {
+            'target': sushy.BOOT_SOURCE_TARGET_PXE,
+            'enabled': sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
+            'mode': sushy.BOOT_SOURCE_MODE_BIOS,
+        }
+        fake_system = mock.Mock(boot=boot_attribute)
         fake_system.set_system_boot_options.side_effect = (
             sushy.exceptions.SushyError)
         mock_get_system.return_value = fake_system
@@ -387,6 +422,27 @@ class RedfishManagementTestCase(db_base.DbTestCase):
                 mode=boot_modes.UEFI)
             mock_get_system.assert_called_once_with(task.node)
 
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_set_boot_mode_unsupported(self, mock_get_system, mock_sushy):
+        boot_attribute = {
+            'target': sushy.BOOT_SOURCE_TARGET_PXE,
+            'enabled': sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
+        }
+        fake_system = mock.Mock(boot=boot_attribute)
+        error = sushy.exceptions.BadRequestError('PATCH', '/', mock.Mock())
+        fake_system.set_system_boot_options.side_effect = error
+        mock_get_system.return_value = fake_system
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            self.assertRaisesRegex(
+                exception.UnsupportedDriverExtension,
+                'does not support set_boot_mode',
+                task.driver.management.set_boot_mode, task, boot_modes.UEFI)
+            fake_system.set_system_boot_options.assert_called_once_with(
+                mode=boot_modes.UEFI)
+            mock_get_system.assert_called_once_with(task.node)
+
     @mock.patch.object(redfish_utils, 'get_system', autospec=True)
     def test_get_boot_mode(self, mock_get_system):
         boot_attribute = {
@@ -1446,3 +1502,23 @@ class RedfishManagementTestCase(db_base.DbTestCase):
             self.assertRaises(
                 exception.UnsupportedDriverExtension,
                 task.driver.management.clear_secure_boot_keys, task)
+
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_get_mac_addresses_success(self, mock_get_system):
+        expected_properties = {'00:11:22:33:44:55': 'enabled'}
+
+        self.init_system_mock(mock_get_system.return_value)
+
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            self.assertEqual(expected_properties,
+                             task.driver.management.get_mac_addresses(task))
+
+    @mock.patch.object(redfish_utils, 'get_system', autospec=True)
+    def test_get_mac_addresses_no_ports_found(self, mock_get_system):
+
+        system_mock = self.init_system_mock(mock_get_system.return_value)
+        system_mock.ethernet_interfaces.summary = None
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            self.assertIsNone(task.driver.management.get_mac_addresses(task))
