@@ -20,11 +20,10 @@ import datetime
 import os
 import tempfile
 import time
+from unittest import mock
 import uuid
 
-import mock
 from oslo_utils import uuidutils
-import six
 
 from ironic.common import exception
 from ironic.common import image_service
@@ -63,6 +62,24 @@ class TestImageCacheFetch(base.TestCase):
             None, self.uuid, self.dest_path, True)
         self.assertFalse(mock_clean_up.called)
 
+    @mock.patch.object(image_cache, '_fetch', autospec=True)
+    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
+    @mock.patch.object(image_cache.ImageCache, '_download_image',
+                       autospec=True)
+    def test_fetch_image_no_master_dir_memory_low(self,
+                                                  mock_download,
+                                                  mock_clean_up,
+                                                  mock_fetch):
+        mock_fetch.side_effect = exception.InsufficentMemory
+        self.cache.master_dir = None
+        self.assertRaises(exception.InsufficentMemory,
+                          self.cache.fetch_image,
+                          self.uuid, self.dest_path)
+        self.assertFalse(mock_download.called)
+        mock_fetch.assert_called_once_with(
+            None, self.uuid, self.dest_path, True)
+        self.assertFalse(mock_clean_up.called)
+
     @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
     @mock.patch.object(image_cache.ImageCache, '_download_image',
                        autospec=True)
@@ -168,8 +185,7 @@ class TestImageCacheFetch(base.TestCase):
                        autospec=True)
     def test_fetch_image_not_uuid(self, mock_download, mock_clean_up):
         href = u'http://abc.com/ubuntu.qcow2'
-        href_encoded = href.encode('utf-8') if six.PY2 else href
-        href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href_encoded))
+        href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href))
         master_path = ''.join([os.path.join(self.master_dir, href_converted),
                                '.converted'])
         self.cache.fetch_image(href, self.dest_path)
@@ -184,8 +200,7 @@ class TestImageCacheFetch(base.TestCase):
     def test_fetch_image_not_uuid_no_force_raw(self, mock_download,
                                                mock_clean_up):
         href = u'http://abc.com/ubuntu.qcow2'
-        href_encoded = href.encode('utf-8') if six.PY2 else href
-        href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href_encoded))
+        href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href))
         master_path = os.path.join(self.master_dir, href_converted)
         self.cache.fetch_image(href, self.dest_path, force_raw=False)
         mock_download.assert_called_once_with(
@@ -223,6 +238,14 @@ class TestImageCacheFetch(base.TestCase):
         self.assertEqual(2, mock_link.call_count)
         self.assertTrue(mock_log.error.called)
 
+    @mock.patch.object(image_cache, '_fetch', autospec=True)
+    def test__download_image_raises_memory_guard(self, mock_fetch):
+        mock_fetch.side_effect = exception.InsufficentMemory
+        self.assertRaises(exception.InsufficentMemory,
+                          self.cache._download_image,
+                          self.uuid, self.master_path,
+                          self.dest_path)
+
 
 @mock.patch.object(os, 'unlink', autospec=True)
 class TestUpdateImages(base.TestCase):
@@ -736,8 +759,12 @@ class TestFetchCleanup(base.TestCase):
     @mock.patch.object(images, 'converted_size', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(images, 'force_raw_will_convert', autospec=True,
+                       return_value=True)
     @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
-    def test__fetch(self, mock_clean, mock_raw, mock_fetch, mock_size):
+    def test__fetch(
+            self, mock_clean, mock_will_convert, mock_raw, mock_fetch,
+            mock_size):
         mock_size.return_value = 100
         image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
         mock_fetch.assert_called_once_with('fake', 'fake-uuid',
@@ -745,3 +772,49 @@ class TestFetchCleanup(base.TestCase):
         mock_clean.assert_called_once_with('/foo', 100)
         mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
                                          '/foo/bar.part')
+        mock_will_convert.assert_called_once_with('fake-uuid', '/foo/bar.part')
+
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(images, 'force_raw_will_convert', autospec=True,
+                       return_value=False)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_already_raw(
+            self, mock_clean, mock_will_convert, mock_raw, mock_fetch,
+            mock_size):
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False)
+        mock_clean.assert_not_called()
+        mock_size.assert_not_called()
+        mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
+                                         '/foo/bar.part')
+        mock_will_convert.assert_called_once_with('fake-uuid', '/foo/bar.part')
+
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(images, 'force_raw_will_convert', autospec=True,
+                       return_value=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_estimate_fallback(
+            self, mock_clean, mock_will_convert, mock_raw, mock_fetch,
+            mock_size):
+        mock_size.side_effect = [100, 10]
+        mock_clean.side_effect = [exception.InsufficientDiskSpace(), None]
+
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False)
+        mock_size.assert_has_calls([
+            mock.call('/foo/bar.part', estimate=False),
+            mock.call('/foo/bar.part', estimate=True),
+        ])
+        mock_clean.assert_has_calls([
+            mock.call('/foo', 100),
+            mock.call('/foo', 10),
+        ])
+        mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
+                                         '/foo/bar.part')
+        mock_will_convert.assert_called_once_with('fake-uuid', '/foo/bar.part')
