@@ -611,6 +611,9 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
         )
         self.node.driver_internal_info['agent_url'] = 'http://1.2.3.4:1234'
         dhcp_factory.DHCPFactory._dhcp_provider = None
+        # Memory checks shoudn't apply here as they will lead to
+        # false positives for unit testing in CI.
+        self.config(minimum_memory_warning_only=True)
 
     def test_get_properties(self):
         with task_manager.acquire(self.context, self.node.uuid,
@@ -1000,10 +1003,9 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
         do_agent_iscsi_deploy_mock.return_value = deployment_uuids
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.cached_agent_client = mock.sentinel.agent_client
             task.driver.deploy.write_image(task)
             do_agent_iscsi_deploy_mock.assert_called_once_with(
-                task, mock.sentinel.agent_client)
+                task, task.driver.deploy._client)
             self.assertEqual(
                 task.node.driver_internal_info['deployment_uuids'],
                 deployment_uuids)
@@ -1082,6 +1084,23 @@ class ISCSIDeployTestCase(db_base.DbTestCase):
             set_boot_device_mock.assert_called_once_with(
                 mock.ANY, task, device=boot_devices.DISK, persistent=True)
 
+    @mock.patch.object(iscsi_deploy, 'do_agent_iscsi_deploy', autospec=True)
+    @mock.patch.object(utils, 'is_memory_insufficent', autospec=True)
+    def test_write_image_out_of_memory(self, mock_memory_check,
+                                       mock_do_iscsi_deploy):
+        mock_memory_check.return_value = True
+        self.node.provision_state = states.DEPLOYWAIT
+        self.node.target_provision_state = states.ACTIVE
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            value = task.driver.deploy.write_image(task)
+            self.assertEqual(states.DEPLOYWAIT, value)
+            self.assertEqual(states.DEPLOYWAIT, task.node.provision_state)
+            self.assertIn('skip_current_deploy_step',
+                          task.node.driver_internal_info)
+        self.assertTrue(mock_memory_check.called)
+        self.assertFalse(mock_do_iscsi_deploy.called)
+
     @mock.patch.object(manager_utils, 'restore_power_state_if_needed',
                        autospec=True)
     @mock.patch.object(manager_utils, 'power_on_node_if_needed', autospec=True)
@@ -1239,7 +1258,7 @@ class CleanUpFullFlowTestCase(db_base.DbTestCase):
         mock_get_deploy_image_info.return_value = {}
 
         with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
+                                  shared=False) as task:
             task.driver.deploy.clean_up(task)
             mock_get_instance_image_info.assert_called_with(task,
                                                             ipxe_enabled=False)
@@ -1581,8 +1600,7 @@ class PhysicalWorkTestCase(tests_base.TestCase):
             'iscsiadm',
             '-m', 'node',
             '-S',
-            run_as_root=True,
-            check_exit_code=[0])
+            run_as_root=True)
 
     @mock.patch.object(utils, 'execute', autospec=True)
     def test_force_iscsi_lun_update(self, mock_exec):
@@ -1593,8 +1611,7 @@ class PhysicalWorkTestCase(tests_base.TestCase):
             '-m', 'node',
             '-T', iqn,
             '-R',
-            run_as_root=True,
-            check_exit_code=[0])
+            run_as_root=True)
 
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(iscsi_deploy, 'verify_iscsi_connection', autospec=True)
@@ -1618,7 +1635,6 @@ class PhysicalWorkTestCase(tests_base.TestCase):
             '-T', iqn,
             '--login',
             run_as_root=True,
-            check_exit_code=[0],
             attempts=5,
             delay_on_retry=True)
 
@@ -1759,7 +1775,6 @@ class PhysicalWorkTestCase(tests_base.TestCase):
             '-T', iqn,
             '--login',
             run_as_root=True,
-            check_exit_code=[0],
             attempts=5,
             delay_on_retry=True)
 
