@@ -13,8 +13,9 @@
 #    under the License.
 
 import datetime
+import types
+from unittest import mock
 
-import mock
 from testtools.matchers import HasLength
 
 from ironic.common import exception
@@ -31,8 +32,10 @@ class TestVolumeConnectorObject(db_base.DbTestCase,
         super(TestVolumeConnectorObject, self).setUp()
         self.volume_connector_dict = db_utils.get_test_volume_connector()
 
-    @mock.patch('ironic.objects.VolumeConnector.get_by_uuid')
-    @mock.patch('ironic.objects.VolumeConnector.get_by_id')
+    @mock.patch('ironic.objects.VolumeConnector.get_by_uuid',
+                spec_set=types.FunctionType)
+    @mock.patch('ironic.objects.VolumeConnector.get_by_id',
+                spec_set=types.FunctionType)
     def test_get(self, mock_get_by_id, mock_get_by_uuid):
         id = self.volume_connector_dict['id']
         uuid = self.volume_connector_dict['uuid']
@@ -81,7 +84,8 @@ class TestVolumeConnectorObject(db_base.DbTestCase,
                 self.context, limit=4, sort_key='uuid', sort_dir='asc')
 
             mock_get_list.assert_called_once_with(
-                limit=4, marker=None, sort_key='uuid', sort_dir='asc')
+                limit=4, marker=None, sort_key='uuid', sort_dir='asc',
+                project=None)
             self.assertThat(volume_connectors, HasLength(1))
             self.assertIsInstance(volume_connectors[0],
                                   objects.VolumeConnector)
@@ -95,7 +99,8 @@ class TestVolumeConnectorObject(db_base.DbTestCase,
                 self.context, limit=4, sort_key='uuid', sort_dir='asc')
 
             mock_get_list.assert_called_once_with(
-                limit=4, marker=None, sort_key='uuid', sort_dir='asc')
+                limit=4, marker=None, sort_key='uuid', sort_dir='asc',
+                project=None)
             self.assertEqual([], volume_connectors)
 
     def test_list_by_node_id(self):
@@ -108,7 +113,8 @@ class TestVolumeConnectorObject(db_base.DbTestCase,
                 self.context, node_id, limit=10, sort_dir='desc')
 
             mock_get_list_by_node_id.assert_called_once_with(
-                node_id, limit=10, marker=None, sort_key=None, sort_dir='desc')
+                node_id, limit=10, marker=None, sort_key=None, sort_dir='desc',
+                project=None)
             self.assertThat(volume_connectors, HasLength(1))
             self.assertIsInstance(volume_connectors[0],
                                   objects.VolumeConnector)
